<?php
/*
 * lt_exercise.php - Vips plugin for Stud.IP
 * Copyright (c) 2003-2005  Erik Schmitt, Philipp Hgelmeyer
 * Copyright (c) 2005-2006  Christa Deiwiks
 * Copyright (c) 2006-2007  Elmar Ludwig, Martin Schrder
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

require_once 'exercises/exercise.php';

/**
 * XXX detailed description
 *
 * @author    XXX
 * @copyright XXX
 * @abstract
 */
class lt_exercise extends Exercise {
    
//      var $g_numberOfAnswers;
//      var $g_ExerciseName        = "Freie Antwort";
//      var $g_ExerciseDescription = "Freie Antwort";
//  
    function lt_exercise() {
    }
    
    
    
    /**
     *    Evaluates a solution.
     *
     *    @abstract
     *    @access public
     *    @returns int (from 0 to 100 ) as an % value
     */
    
    function evaluate ($exercise,$solution)
    {
        $dom_xml = $this->getDomXML($exercise);
        $answers = $dom_xml->get_elements_by_tagname("Answer");
        
        $answer_array = array();
        $correct_array = array();
        
        foreach ($answers as $node) {
            $answer_array[]  = utf8_decode($node->get_content());
            $correct_array[] = $node->get_attribute("Correct");
        }
        
        $answer_distance = $this->getTagsFromXML($exercise, "AnswerDistance");
        $answer_distance = $answer_distance[0];
        
        $student_array = $this->getTagsFromXML($solution, "answer");
        $studentSolution = $student_array[0];
        
        $similarity = 0;
        $safe = false;
        $studentSolution = normalizeText($studentSolution);
        
        if ($studentSolution == '')
        {
            $m_return[1] = 0;       // no points
            $m_return[2] = 1;
            return $m_return;
        }
        
        for ($i = 0; $i < count($answer_array); ++$i)
        {
            $musterLoesung = $answer_array[$i];
            $musterLoesung = normalizeText($musterLoesung);
            $divisor = max(strlen($musterLoesung), strlen($studentSolution));
            $similarity_temp = 0;
            
            if ($musterLoesung == $studentSolution)
            {
                $similarity_temp = 1;
            }
            else if ($answer_distance == 1)  // Levenshtein-Zeichen
            {
                $levenshtein = levenshtein($studentSolution, $musterLoesung) / $divisor;
                $similarity_temp = 1 - $levenshtein;
            }
            else if ($answer_distance == 2)  // Soundex-Aussprache
            {
                $levenshtein = levenshtein(soundex($musterLoesung), soundex($studentSolution));         
                
                if (soundex($musterLoesung) == soundex($studentSolution))
                    $similarity_temp = 0.8;
                else if ($levenshtein == 1)
                    $similarity_temp = 0.6;
                else if ($levenshtein == 2)
                    $similarity_temp = 0.4;
                else if ($levenshtein == 3)
                    $similarity_temp = 0.2;
                else  // $levenshtein == 4
                    $similarity_temp = 0;
            }
            
            if ($correct_array[$i] == 1)  // correct
            {
                if ($similarity_temp > $similarity)
                {
                    $similarity = $similarity_temp;
                    $safe = $similarity_temp == 1;
                }
            }
            else if ($correct_array[$i] == 0.5)  // half correct
            {
                if ($similarity_temp > $similarity)
                {
                    $similarity = $similarity_temp * 0.5;
                    $safe = $similarity_temp == 1;
                }
            }
            else if ($similarity_temp == 1)  // false
            {
                $m_return[1] = 0;       // no points
                $m_return[2] = 1;
                return $m_return;
            }
        }
        
        $m_return[1] = $similarity * 100;       // percent
        $m_return[2] = $safe;
        return $m_return;
    }
    
    
    
    /**
     *    This function gens a Solution, that is stored to the database.
     *    solution_id.
     *
     *    @access public
     *    @returns XML contains the solution coded in an XML-string
     */
    
    function genSolution($exercise_id, $p_number_of_answers) {
        
        $msg = '<solution id="'.$exercise_id.'">';
        
        //same as tb_exercise
        $msg .= '<answer id="0">';
        $msg .= '<body>'.htmlspecialchars(getExternalParam("answer_0")).'</body>';
        $msg .= '</answer>';
                
        $msg.= '</solution>';
        return ($msg);
    }
    
    
    
    /**
     *    Generate the XML-input for the database for an exercise.
     *
     *    @abstract
     *    @access public
     *    @returns bool
     */
    
    function getExerciseConstruct() {
        
        /*************************************************************************/
        /* Get data
        /*************************************************************************/
        
        $m_exercise_language = "de";
        
        $m_exercise_type     = getExternalParam("exercise_type");
        $m_exercise_question = getExternalParam("exercise_question");
        $m_exercise_name     = getExternalParam("exercise_name");
        $m_exercise_hint     = getExternalParam("exercise_hint");
        $m_exercise_comment  = getExternalParam("exercise_comment");
        
        $m_exercise_pic_url  = getExternalParam("exercise_pic_url");  //??
        $m_provides          = getExternalParam("provides");  //??
        $m_depends           = getExternalParam("depends"); //??
        $m_optional_depends  = getExternalParam("optional_depends"); //??
        $m_number_of_answers = getExternalParam("number_of_answers");
        
        $m_answer_distance   = getExternalParam("answer_distance");             // method of distance calculation
        
        $i = 0;
        do {
            $correct_array[$i] = getExternalParam("correct_".$i);
            $answer_array[$i]  = getExternalParam("answer_".$i);
            $i++;
        } while ($i < $m_number_of_answers);
        
        /*************************************************************************/
        /* Construct XML
        /*************************************************************************/
        
        $xmlop  = "<TestItem ID=\"" . $m_exercise_type . "\" Type=\"MultipleChoice\" Language=\"" . $m_exercise_language . "\">";
        $xmlop .= "<Name>".htmlspecialchars($m_exercise_name)."</Name>";
        $xmlop .= "<Hint>".htmlspecialchars($m_exercise_hint)."</Hint>";
        $xmlop .= "<CommentField Visible=\"".$m_exercise_comment."\"></CommentField>";
        $xmlop .= "<AnswerDistance>".htmlspecialchars($m_answer_distance)."</AnswerDistance>";
        $xmlop .= "<PictureURL>".htmlspecialchars($m_exercise_pic_url)."</PictureURL>";
        $xmlop .= "<Provides>".htmlspecialchars($m_provides)."</Provides>";
        $xmlop .= "<Depends>".htmlspecialchars($m_depends)."</Depends>";
        $xmlop .= "<OptionalDepends>".htmlspecialchars($m_optional_depends)."</OptionalDepends>";
        $xmlop .= "<Question>";
        $xmlop .= "<Paragraph>".htmlspecialchars($m_exercise_question)."</Paragraph>";
        $xmlop .= "</Question>";
        
        for ($i = 0; $i < count($answer_array); $i++) {
            if ($answer_array[$i])  // store only meaningful answers
                $xmlop .= '<Answer Correct="'.$correct_array[$i].'">'.htmlspecialchars($answer_array[$i]).'</Answer>';
        }
        $xmlop .= "</TestItem>";
        
        /*************************************************************************/
        /* Return XML
        /*************************************************************************/
        
        return $xmlop;
    }

    /**
     * Export this exercise to plain text format.
     */
    function exportText ($exercise_xml)
    {
        $result = parent::exportText($exercise_xml, 'Frage');

        $dom_xml = $this->getDomXML($exercise_xml);
        $answers = $dom_xml->get_elements_by_tagname('Answer');

        foreach ($answers as $node)
        {
            $answer_array[] = utf8_decode($node->get_content());
            $correct_array[] = $node->get_attribute('Correct');
        }

        $answer_distance = $this->getTagsFromXML($exercise_xml, 'AnswerDistance');
        $answer_distance = $answer_distance[0];

        if ($answer_distance == 1)
            $result .= "Abgleich: Levenshtein\n";
        else if ($answer_distance == 2)
            $result .= "Abgleich: Soundex\n";

        for ($i = 0; $i < count($answer_array); ++$i)
        {
            if ($correct_array[$i] == '1')
                $result .= '+';
            else if ($correct_array[$i] == '0.5')
                $result .= '~';

            $result .= 'Antwort: '.$answer_array[$i]."\n";
        }

        return $result;
    }
}
?>
