<?php
/*
 *  $Id: IndexInfo.php,v 1.3 2005/03/01 13:56:43 micha Exp $
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many individuals
 * and is licensed under the LGPL. For more information please see
 * <http://creole.phpdb.org>.
 */

/**
 * Represents an index.
 *
 * @author    Hans Lellelid <hans@xmpl.org>
 * @author    Michael Aichler <aichler@mediacluster.de>
 * @version   $Revision: 1.3 $
 * @package   creole.metadata
 */
class IndexInfo
{
  /**
  * Name of the index
  * @var string
  */
  var $name;
  /**
  * Columns in this index
  * @var array
  */
  var $columns = array();
  /**
  * Uniqueness flag
  * @var bool
  */
  var $isUnique = false;
  /**
  * Additional vendor specific information
  * @var array
  */
  var $vendorSpecificInfo = array();


  /**
  * IndexInfo constructor.
  *
  * @param string $name Index name
  * @param array $vendorInfo Vendor specific info array
  */
  function IndexInfo($name, $isUnique = false, $vendorInfo = array())
  {
    $this->name = $name;
    $this->isUnique = $isUnique;
    $this->vendorSpecificInfo = $vendorInfo;
  }

  /**
  * Gets the name of this index.
  *
  * @return string
  */
  function getName()
  {
    return $this->name;
  }

  /**
  * Returns whether this index is unique.
  *
  * @return bool
  */
  function isUnique()
  {
    return $this->isUnique;
  }

  /**
  * Gets added columns.
  *
  * @return array
  */
  function & getColumns()
  {
    return $this->columns;
  }

  /**
  * Get vendor specific optional information for this index.
  *
  * @return array
  */
  function getVendorSpecificInfo()
  {
    return $this->vendorSpecificInfo;
  }

  /**
  * Adds a column to this index.
  *
  * @param ColumnInfo $column
  * @return void
  */
  function addColumn(&$column)
  {
    Creole::typeHint($column, 'ColumnInfo', 'IndexInfo', 'addColumn', 1);
    $this->columns[] =& $column;
  }

  /**
  * Returns a string representation for this index.
  *
  * @return string
  */
  function toString()
  {
    return $this->name;
  }

}
