<?php
/*
 * vips_assignments.inc.php - Vips plugin for Stud.IP
 * Copyright (c) 2003-2005  Erik Schmitt, Philipp Hgelmeyer
 * Copyright (c) 2005-2006  Christa Deiwiks
 * Copyright (c) 2006-2007  Elmar Ludwig, Martin Schrder
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

/**
 * 
 * Core file of Vips (I guess...). Handles sheets and exams (that depends on $p_action), 
 * everything from creating exercises to assigning assignments to blocks
 * 
 * @author: Christa Deiwiks <cdeiwiks@uos.de>
 */

function vips_assignments ($action, $subaction)
{
    global $g_outlet,
           $g_assignment_table,
           $g_inAssignment_table,
           $g_solution_table,
           $g_solution_points_table,
           $g_title_s,
           $g_title_p;

    if ($action == 'sheets') {
        $g_assignment_table      = 'vips_uebungsblatt'; //assign necessary variables for exercise sheets
        $g_title_s               = _('bungsblatt');
        $g_title_p               = _('bungsbltter');
        $g_inAssignment_table    = 'vips_inUebungsblatt';
        $g_solution_table        = 'vips_T_Loesung';
        $g_solution_points_table = 'vips_T_Loesung_Punkte';
    }
    else {  // $action == 'exams'
        $g_assignment_table      = 'vips_klausur'; //assign necessary variables for exams
        $g_title_s               = _('Klausur');
        $g_title_p               = _('Klausuren');
        $g_inAssignment_table    = 'vips_inKlausur';
        $g_solution_table        = 'vips_K_Loesung';
        $g_solution_points_table = 'vips_K_Loesung_Punkte';
    }

    /****************************/
    /* LECTURER/TUTOR FUNCTIONS */
    /****************************/
    if ($g_outlet->status == 'lecturer' || $g_outlet->status == 'tutor') {
        if ($subaction == 'list_assignments') {
            list_assignments($action);
        }
        else if ($subaction == 'list_assignments_stud') {
            list_assignments_stud($action);
        }
        else if ($subaction == 'new_assignment') {
            new_assignment($action);
        }
        else if ($subaction == 'copy_assignment') {
            copy_assignment($action);
        }
        else if ($subaction == 'store_assignment') {
            store_assignment($action);
        }
        else if ($subaction == 'store_exercise') {
            store_exercise($action);
        }
        else if ($subaction == 'edit_assignment') {
            edit_assignment($action);
        }
        else if ($subaction == 'edit_exercise') {
            edit_exercise($action);
        }
        else if ($subaction == 'delete_assignment') {
            delete_assignment($action);
        }
        else if ($subaction == 'delete_exercise') {
            delete_exercise($action);
        }
        else if ($subaction == 'delete_exercise_ajax') {
            delete_exercise_ajax($action);
        }
        else if ($subaction == 'move_exercise') {
            move_exercise($action);
        }
        else if ($subaction == 'move_exercise_ajax') {
            move_exercise_ajax($action);
        }
        else if ($subaction == 'pick_semester_ajax') {
            pick_semester_ajax($action);
        }
        else if ($subaction == 'pick_test_ajax') {
            pick_test_ajax($action);
        }
        else if ($subaction == 'preview_exercise') {
            preview_exercise($action);
        }
        else if ($subaction == 'copy_exercise_ajax') {
            copy_exercise($action, true);
        }
        else if ($subaction == 'move_assignment') {    //???
            move_assignment($action);
        }
        else if ($subaction == 'stopgo_assignment') {
            stopgo_assignment($action);
        }
        else if ($subaction == 'start_assignment') {
            start_assignment($action);
        }
        else if ($subaction == 'show_exercise') {
            show_exercise($action);
        }
        else if ($subaction == 'show_solution_to_lecturer') {
            show_solution_to_lecturer($action);
        }
        else if ($subaction == 'submit_exercise') {
            submit_exercise($action);
        }
        else if ($subaction == 'add_block') {
            add_block($action);
        }
        else if ($subaction == 'assignment_options') {
            assignment_options($action);
        }
        else if ($subaction == 'delete_solutions') {
            delete_solutions($action);
        }
        else {
            list_assignments($action);
        }
    }

    /*********************/
    /* STUDENT FUNCTIONS */
    /*********************/
    else {
        if ($subaction == 'show_exercise') {
            show_exercise($action);
        }
        else if ($subaction == 'submit_exercise') {
            submit_exercise($action);
        }
        else if ($subaction == 'delete_solutions') {
            delete_solutions($action);
        }
        else {
            list_assignments_stud($action);
        }
    }
}

#####################################
#####################################
#                                   #
#          Student Methods          #
#                                   #
#####################################
#####################################

/**
 * 
 * EXAM
 * Only possible if exam is selftest: Deletes all the solutions of a student to enable him/her
 * to redo it.
 * 
 */
function delete_solutions($p_action) {
        global $vips_db,
               $g_outlet,
               $g_inAssignment_table,
               $g_assignment_table,
               $g_solution_points_table,
               $g_solution_table;
        
        $assignment_id = getExternalParam("assignment_id");
        
        if (getExternalParam('cancel') || getExternalParam('cancel_x') != '')
        {
                list_assignments_stud($p_action);
        }
        else if (getExternalParam('delete_solutions_confirmed') || getExternalParam('delete_solutions_confirmed_x') != '')
        {
                $vips_db->executeUpdate("UPDATE $g_solution_table
                                        SET isLastSolution = 'f'
                                        WHERE Assignmentid = '$assignment_id'
                                        AND Userid = '$g_outlet->userID'");

                $vips_db->executeUpdate("DELETE FROM vips_aufgaben_zeit
                                        WHERE Kurs              = '$g_outlet->courseID'
                                        AND $g_assignment_table = '$assignment_id'
                                        AND user_id             = '$g_outlet->userID'");

                $message['mode']        = 'msg';
                $message['message']     = 'Die Lsungen wurden gelscht!';
                $_POST['message']       = $message;
                list_assignments_stud($p_action);
        }
        else
        {
                $_POST['delete_assignment_id']  = $assignment_id;
                $_POST['dialog']                = 'delete_solutions_confirm';
                list_assignments_stud($p_action);
        }
}

/**
 * SHEETS/EXAMS
 * 
 * Is called when the submit button at the bottom of an exercise is called.
 * If there is already a solution of this exercise by the same user or same group,
 * a dialog pops up to confirm the submission. On database-level: EVERY solution is stored
 * (even the unconfirmed ones), with the last solution being marked as last.
 * 
 */

function submit_exercise($p_action) {
        global $vips_db,
                   $g_outlet,
                   $g_inAssignment_table,
                   $g_assignment_table,
                   $g_solution_table;

        $exercise_type                  = getExternalParam("exercise_type");
        $exercise_id                    = getExternalParam("exercise_id");
        $exercise_position              = getExternalParam("exercise_position");
        $assignment_id                  = getExternalParam("assignment_id");

        $student_comment                = getExternalParam("student_comment");
        $number_of_answers              = getExternalParam("number_of_answers");

        if (getExternalParam("in_rnd_exercise"))
            $exercise_id = getExternalParam("rnd_exercise_id");

        $message = array();

        $pl_out = '';

        $solved = getExternalParam("solved");
        $forced = getExternalParam("forced");


        ##################################################################
        # in case student solution is submitted by tutor or lecturer     #
        # (can happen if the student submits his/her solution by email)  #
        ##################################################################
        
        $solver_user_id = getExternalParam("solver_user_id");

        if ($solver_user_id == '' || $g_outlet->status == 'student') {
            $solver_user_id = $g_outlet->userID;
        }
        
        // from here on we only use $solver_user_id and not $g_outlet->userID

        ###################################
        #  get info of Selbsttest (EXAMS) #
        ###################################

        $result = $vips_db->executeQuery("SELECT * FROM $g_assignment_table
                                                  WHERE ID  = '$assignment_id'
                                                  ");
        if ($result->next()) {
                $entry = $result->getRow();
                $selftest = $entry['Selbsttest'];
                $ende = $entry['Ende'];
        }

        ############################
        # Checks before submission #
        ############################
        
        // if time has elapsed
        if ((getRemainingMinutes($assignment_id) <= -2 || date('Y-m-d H:i:s') > $ende) &&
            $selftest == 0 && $p_action == 'exams' &&
            $g_outlet->status == 'student')
        {
                $message['mode']    = 'error';
                $message['message'] = 'Ihre Zeit ist leider abgelaufen! Ihre Lsung wurde gespeichert und mit dem Abgabezeitpunkt versehen.';
                $_POST['message']   = $message;
                //list_assignments_stud($p_action); #vvvvvv continue to store vvvvvvvvv
        }
        // if nobody is logged on
        else if ($g_outlet->status == 'nobody')
        {
                $message['mode']    = 'error';
                $message['message'] = 'Ihre Session ist leider abgelaufen. Bitte loggen Sie sich erneut ein!';
                $_POST['message']   = $message;
                list_assignments_stud($p_action);
        }

        if ($g_outlet->status != 'nobody') {
                /*
                 * special case for Prolog exercises:
                 * submit_exercise() can be called without actually wanting to submit a solution but to
                 * send a solution to the PROLOG server
                 */
                if (getExternalParam("pl_exercise_mode_query") || getExternalParam("pl_exercise_mode_querynext") ||
                    getExternalParam('pl_exercise_mode_upload') || getExternalParam('pl_exercise_mode_upload_x') != '' ||
                    getExternalParam('pl_exercise_mode_download') || getExternalParam('pl_exercise_mode_download_x') != '')
                {
                        require_once("exercises/pl_exercise.php");

                        $o_exercise = new pl_exercise;

                        //$solution_XML         = $o_exercise->genSolution($exercise_id, $number_of_answers); //needed?

                        $answer_0 = getExternalParam('answer_0');

                        if (getExternalParam('pl_exercise_mode_query')) {
                                $m_reset        = TRUE;
                                $pl_out         = $o_exercise->evaluateQuery($solver_user_id,$g_outlet->courseID);
                        }
                        elseif (getExternalParam('pl_exercise_mode_querynext')) {
                                $pl_count       = getExternalParam('pl_count')+1;
                                $pl_out         = $o_exercise->evaluateQuery($solver_user_id,$g_outlet->courseID,$pl_count);
                        }
            
            elseif (getExternalParam('pl_exercise_mode_upload') || getExternalParam('pl_exercise_mode_upload_x') != '') {
                $answer_0 = $o_exercise->uploadSolution();
            }
            
            elseif (getExternalParam('pl_exercise_mode_download') || getExternalParam('pl_exercise_mode_download_x') != '') {
                $sql = "SELECT `Name` FROM `vips_aufgabe` WHERE `ID` = '$exercise_id'";
                $result = $vips_db->executeQuery($sql);
                if ($result->next()) {
                    $exercise_name = preg_replace('/\W/', '_', $result->getString('Name'));
                }
                $o_exercise->downloadSolution($exercise_name);
            }
                        $_POST['assignment_id']     = $assignment_id;
                        $_POST['exercise_position'] = $exercise_position;
                        $_POST['answer_0']          = stripslashes($answer_0);
                        $_POST['pl_query']          = stripslashes(htmlspecialchars(getExternalParam('pl_query')));
                        $_POST['pl_out']            = stripslashes($pl_out);
                        $_POST['pl_count']          = $pl_count;
                        $_POST['message']           = $message;
                        unset($_GET['next']);                             //to prevent show_exercise to display the next question
                        unset($_GET['prev']);
                        
                        show_exercise($p_action);
                }  // end prolog-specific
                // default: store exercise
                else
                {
                        $flag = false;   //is set to TRUE further down in the confirmation dialog (just internal communication...)
                        $new_solution_id = 0;

                        /* if an exercise has been submitted */
                        if (getExternalParam('submit_exercise') || getExternalParam('submit_exercise_x') != '' || $forced)
                        {
                                require_once("exercises/$exercise_type.php");

                                $o_exercise     = new $exercise_type;
                                $solution_XML   = $o_exercise->genSolution($exercise_id, $number_of_answers);

                                //get group id if applicable
                                $result2 = $vips_db->executeQuery(
                                                "       SELECT vips_inGruppe.* FROM
                                                                vips_inGruppe,
                                                                vips_gruppe
                                                        WHERE user_id = '$solver_user_id'
                                                        AND  vips_gruppe.Gruppenid                      = vips_inGruppe.group_id
                                                        AND  vips_gruppe.Kursid                         = '$g_outlet->courseID'
                                                        AND  vips_inGruppe.end IS NULL");
                                                        
                                $group_id = 0;
                                if ($result2->next()) {
                                        $group = $result2->getRow();
                                        $group_id = $group['group_id'];
                                }
                                
                                $current_ip = $_SERVER['REMOTE_ADDR'];
                                
                                $sql = "INSERT INTO $g_solution_table
                                                SET
                                                        Loesung                 = '$solution_XML',
                                                        Aufgabenid              = '$exercise_id',
                                                        Assignmentid    = '$assignment_id',
                                                        Userid                  = '$solver_user_id',
                                                        Gruppenid               = '$group_id',
                                                        Zeitpunkt               = NOW(),
                                                        isLastSolution  = 'f',
                                                        Anmerkung               = '$student_comment'    
                                                ";
                                //store IP address of current user too
                                if ($p_action == 'exams') 
                                        $sql .= ", IPAdresse    = '$current_ip'";

                                $vips_db->executeUpdate($sql);
                                $vips_idgen = $vips_db->getIdGenerator();
                                $new_solution_id = $vips_idgen->getId();
                                $_POST['new_solution_id'] = $new_solution_id;
                                
                                //if exercise has been solved before, envoke confirmation dialog but keep ID of
                                //new solution "in mind"
                                if ($solved == 1 && $forced == 0 && $g_outlet->status == 'student' && $p_action == 'sheets') {
                                        $dialog = 'overwrite';
                                        $flag = true;
                                        $_POST['dialog'] = $dialog;
                                        $_POST['new_solution_id'] = $new_solution_id;
                                        unset($_GET['next']);
                                        unset($_GET['prev']);
                                        show_exercise($p_action);
                                }
                        }

                        // if the submission has been cancelled (from dialog)
                        if (getExternalParam('cancel') || getExternalParam('cancel_x') != '') 
                        {
                                list_assignments_stud($p_action);
                        }
                        // else store user solution
                        elseif (!$flag)         
                        {
                                //get group id if applicable
                                $result2 = $vips_db->executeQuery(
                                                "       SELECT vips_inGruppe.* FROM
                                                                vips_inGruppe,
                                                                vips_gruppe
                                                        WHERE user_id = '$solver_user_id'
                                                        AND  vips_gruppe.Gruppenid                      = vips_inGruppe.group_id
                                                        AND  vips_gruppe.Kursid                         = '$g_outlet->courseID'
                                                        AND  vips_inGruppe.end IS NULL");
                                $group_id = 0;
                                if ($result2->next()) {
                                        $group = $result2->getRow();
                                        $group_id = $group['group_id'];
                                }
                                
                                ################################################
                                # UPDATE old solutions                         #
                                # (set all solutions of this user and exercise #
                                # on isLastSolution='f')                       #
                                ################################################ 
                                if ($p_action == 'exams') {   
                                        $sql = "UPDATE $g_solution_table
                                                SET isLastSolution      = 'f'
                                                WHERE Aufgabenid        = '$exercise_id'
                                                AND   Assignmentid      = '$assignment_id'
                                                AND   Userid            = '$solver_user_id'";
                                } else if ($group_id != 0) {
                                        $sql = "UPDATE $g_solution_table
                                                SET isLastSolution      = 'f'
                                                WHERE Aufgabenid        = '$exercise_id'
                                                AND   Assignmentid      = '$assignment_id'
                                                AND   Gruppenid         = '$group_id'";
                                } else {
                                        $sql = "UPDATE $g_solution_table
                                                SET isLastSolution      = 'f'
                                                WHERE Aufgabenid        = '$exercise_id'
                                                AND   Assignmentid      = '$assignment_id'
                                                AND   Userid            = '$solver_user_id'";
                                }

                                $vips_db->executeUpdate($sql);  
                                
                                ###############################################
                                # UPDATE new solution                                             #
                                # (set new solution on isLastSolution = 't')  #
                                ###############################################
                                
                                $new_solution_id = getExternalParam('new_solution_id');
                                $sql = "UPDATE $g_solution_table
                                        SET isLastSolution      = 't'
                                        WHERE Loesungsid        = '$new_solution_id'";
                                $vips_db->executeUpdate($sql);

                                $_POST['assignment_id']         = $assignment_id;
                                $_POST['exercise_position']     = $exercise_position;
                                $_POST['solved'] = true;
                                $_POST['dialog'] = 'submitted';
                                unset($_GET['next']);
                                unset($_GET['prev']);
                                show_exercise($p_action);
                        }
                }
        } 
}

/**
 * SHEETS/EXAMS
 * 
 * Displays an exercise (from student perspective)
 * 
 * This function is also invoked by a function in vips_solutions.inc.php to show the
 * solution of a student to a lecturer.
 */
function show_exercise($p_action) {
    global $vips_db,
        $g_outlet,
        $g_inAssignment_table,
        $g_assignment_table,
        $g_solution_table,
        $g_solution_points_table,
        $vipsTemplateFactory,
        $vipsPlugin;
    
    $assignment_id     = getExternalParam('assignment_id');
    $exercise_position = getExternalParam('exercise_position'); //identifier! (instead of exercise_id)
    
    // If exercise is in a rnd_exercise, it has no assignment or position! Then: get its id.
    $in_rnd_exercise = getExternalParam("in_rnd_exercise");
    if ($in_rnd_exercise)
    {
        $exercise_id           = getExternalParam("exercise_id");
        $rnd_exercise_id       = getExternalParam("rnd_exercise_id");
        $rnd_exercise_name     = getExternalParam("rnd_exercise_name");
        $rnd_exercise_position = getExternalParam("rnd_exercise_position");
    }
    
    $solver_id      = getExternalParam("solver_id");  // solver is handed over via address line, ie. user is either a lecturer or a student who is looking at corrections
    $solver_user_id = getExternalParam("solver_user_id");  // solver is handed over via address line, lecturer is submitting exercise for student
    $single_solver  = getExternalParam("single_solver"); //user is not participant in any group
    $submitted      = getExternalParam("submitted");    //from edit_solution (for Prolog)
    $print          = getExternalParam('print');
    $samplesolution = getExternalParam("samplesolution");
    $view           = getExternalParam('view');
    $korrektur      = 0;
    
    // this prevents URL-hacking to get the correction view
    if ($solver_id && getExternalParam('subaction') != 'show_exercise')
    {
        $korrektur = 1;
    }
    else
    {
        if ($g_outlet->status == 'student' || !$solver_user_id)
        {
            $solver_user_id = $g_outlet->userID;
        }

        $solver_id = $solver_user_id;                   // "solver_id" is the student himself
        $single_solver = true;

        if ($p_action == 'sheets')
        {
            $query = "SELECT group_id
                FROM vips_inGruppe, vips_gruppe
                WHERE user_id = '".$solver_user_id."'
                    AND vips_gruppe.Kursid = '".$g_outlet->courseID."'
                    AND vips_inGruppe.group_id = vips_gruppe.Gruppenid
                    AND vips_inGruppe.end IS NULL";
            $result = $vips_db->executeQuery($query);
            
            if ($result->next()) {
                $solver_id = $result->get("group_id");  // "solver_id" is the group the student is in
                $single_solver = false;
            }
        }
    }

    $message = $_POST['message'];   //??? needed???
    
    $pl_temp_answer = getExternalParam('answer_0');      //needed for pl_exercise (and other???)
    $pl_temp_query  = getExternalParam('pl_query');      //specific for pl_exercise
    $pl_score       = $_POST['pl_score'];
    $pl_validity    = $_POST['pl_validity'];
    $pl_count       = $_POST['pl_count'];    //specific for pl_exercise
    $pl_out         = $_POST['pl_out'];      //internal communication
    if (!$pl_count) $pl_count = 1;
    
    $dialog          = getExternalParam('dialog');                   //internal communication :-) not from forms
    $new_solution_id = getExternalParam('new_solution_id');
    
    if (getExternalParam('next'))  //$_GET variable in link "Zur naechsten Aufgabe"
    {
        $exercise_position += 1;
    }
    else if (getExternalParam('prev'))
    {
        $exercise_position -= 1;
    }
    
        $solved                 = 0 ;   //getExternalParam('solved');
        $show_solution          = 0 ;

        //get number of exercises in assignment (for footer)


        ################################
        #    get Selbsttest for exams  #
        ################################
        
        $selftest = 0;
        
        // fetch all exercises in the assignment
        $result = $vips_db->executeQuery(
                        "SELECT vips_aufgabe.*
                          FROM  vips_aufgabe, $g_inAssignment_table
                          WHERE $g_inAssignment_table.$g_assignment_table = '$assignment_id'
                          AND vips_aufgabe = vips_aufgabe.ID
                          ");
        $number_of_exercises            = $result->getRecordCount();
        
        $result = $vips_db->executeQuery("SELECT Selbsttest, mc_auswertung
                FROM $g_assignment_table
                WHERE ID  = '$assignment_id'
                ");
        if ($result->next()) {
                $entry = $result->getRow();
                $selftest = $entry['Selbsttest'];
                $mc_auswertung = $entry['mc_auswertung'];
        }

        #####################################
        #       get exercise info           #
        #####################################
        
        // if the display function is called from a rnd_exercise, the exercise
        // has no assignment and no position, just an id...
        if ($in_rnd_exercise)
        {
                // fetch info, type from subexercise, points from rnd_exercise
                $query = "SELECT DISTINCT
                        `vips_aufgabe`.`Aufgabe`
                        , `vips_aufgabe`.`Name`
                        , `vips_aufgabe`.`ID`
                        , `vips_aufgabe`.`URI`
                        , `vips_aufgaben_typen`.`Typenname`
                        , $g_inAssignment_table.Punkte
                FROM
                        `vips_aufgabe`,
                        `vips_aufgaben_typen`,
                        $g_inAssignment_table
                WHERE
                        `vips_aufgabe`.`ID` = '$exercise_id'
                        AND `vips_aufgaben_typen`.`URI` = `vips_aufgabe`.`URI`
                        AND $g_inAssignment_table.vips_aufgabe = $rnd_exercise_id
                ";
        }
        // default
        else
        {
                // fetch exercise info, type and points, determined by assignment and position
                $query = "SELECT DISTINCT
                                vips_aufgabe.*
                                , vips_aufgaben_typen.Typenname
                                , $g_inAssignment_table.Punkte
                        FROM
                                vips_aufgabe
                                , vips_aufgaben_typen
                                , $g_inAssignment_table
                        WHERE
                                $g_inAssignment_table.$g_assignment_table = '$assignment_id'
                                AND $g_inAssignment_table.vips_aufgabe    = vips_aufgabe.ID
                                AND $g_inAssignment_table.Position        = '$exercise_position'
                                AND vips_aufgaben_typen.URI               = vips_aufgabe.URI
                        ";
        }
        
        $result = $vips_db->executeQuery($query);
        $result->next();
        
        $exercise_XML        = $result->get('Aufgabe');
        $exercise_name       = $result->get('Name');
        $exercise_id         = $result->get('ID');
        $exercise_type       = $result->get('URI');
        $exercise_title      = $result->get('Typenname');
        $exercise_max_points = $result->getFloat('Punkte');
        
        // special treatment for rnd_exercise: choose subexercise from XML and get info from db
        if ($exercise_type == "rnd_exercise")
        {
                $in_rnd_exercise = 1;
                $rnd_exercise_id = $exercise_id;
                $rnd_exercise_empty = 1;
                
                require_once("exercises/rnd_exercise.php");
                $rnd0_exercise = new rnd_exercise();
                
                $subexercise_id = $rnd0_exercise->choose_subexercise($exercise_XML, $solver_id, $exercise_id);  // false if no subexercises exist
                
                if ($subexercise_id)
                {
                        $rnd_exercise_empty = 0;
                        
                        $query = "SELECT Aufgabe, Name, ID, vips_aufgabe.URI, Typenname
                                FROM vips_aufgabe
                                        , vips_aufgaben_typen
                                WHERE ID = '".$subexercise_id."'
                                        AND vips_aufgaben_typen.URI = vips_aufgabe.URI
                                ";
                        $result = $vips_db->executeQuery($query);
                        $result->next();
                        
                        $exercise_XML = $result->get("Aufgabe");
                        $exercise_name = $result->get("Name");
                        $exercise_id = $result->get("ID");
                        $exercise_type = $result->get("URI");
                        $exercise_title = $result->get("Typenname");
                }
        }
        
        
        
        ##############################################################
        #    check for ip_address, remaining time and interrupted    #
        ##############################################################
        
        $error = false;
        
        // restrict access for students!
        if ($g_outlet->status == 'student')
        {
                // fetch end time for assignment
                $query = "SELECT Ende
                        FROM $g_assignment_table
                        WHERE ID = '$assignment_id'
                                AND Kurs = '".$g_outlet->courseID."'";
                $result = $vips_db->executeQuery($query);
                $result->next();
                $assignment_end_time = $result->get("Ende");
                
                // if student wants to see the Korrekturansicht, but assignment is still running
                if ($korrektur && !$print && time() < strtotime($assignment_end_time))
                {
                        // if one changes the system time and this way tries to see the
                        // Korrekturansicht, this won't work, as assignment is not
                        // accessible any more after it has run out
                        $error = true;
                        $message["mode"] = 'error';
                        $message["message"] = _("Die Korrekturen drfen erst nach Ablaufen des Aufgabenblattes betrachtet werden")."!";
                        $_POST["message"] = $message;
                        list_assignments_stud($p_action);
                }
                
                if ($p_action == 'exams' && $selftest == 0 && !$korrektur) {
                        if (ip_permitted($assignment_id) == FALSE) {
                                $error = true;
                                $message['mode']        = 'error';
                                $message['message'] = 'Kein Zugriff!';
                                $_POST['message']       = $message;
                                list_assignments_stud($p_action);
                        }
                        elseif (getRemainingMinutes($assignment_id) <= 0) {
                                $error = true;
                                $message['mode']        = 'error';
                                $message['message'] = 'Die Zeit ist leider abgelaufen!';
                                $_POST['message']       = $message;
                                list_assignments_stud($p_action);
                        }
                        elseif (checkInterrupted($assignment_id) == 'y') {
                                $error = true;
                                $message['mode']        = 'error';
                                $message['message'] = 'Die Klausur wurde vom Dozenten unterbrochen!';
                                $_POST['message']       = $message;
                                list_assignments_stud($p_action);
                        }
                }
                else {
                        if (checkInterrupted($assignment_id) == 'y') {
                                $error = true;
                                $message['mode']        = 'error';
                                $message['message'] = 'Das bungsblatt wurde vom Dozenten unterbrochen!';
                                $_POST['message']       = $message;
                                list_assignments_stud($p_action);
                        }
                }
        }

        if (!$error) {
                #################################
                #  get all course participants  #
                #################################
        
                $course_participants = array();
                $result = $vips_db->executeQuery("SELECT seminar_user.*,
                                                                        auth_user_md5.username,
                                                                        auth_user_md5.Vorname, 
                                                                        auth_user_md5.Nachname,
                                                                        auth_user_md5.perms
                                                                FROM seminar_user,
                                                                         auth_user_md5
                                                                WHERE   seminar_user.Seminar_id = '$g_outlet->courseID'
                                                                AND     seminar_user.user_id    = auth_user_md5.user_id
                                                                ORDER BY auth_user_md5.Nachname, auth_user_md5.Vorname");                       
                while ($result->next()) {
                        $course_participant = $result->getRow();
                        $course_participants[] = $course_participant;
                }

                $remainingMinutes = getRemainingMinutes($assignment_id);
                $studentAbgabe    = getAbgabezeitpunkt($assignment_id);  //different for different students!
                
                
                
                ##########################################################
                # get number of exercises in assignment (for navigation) #
                ##########################################################

                $result = $vips_db->executeQuery("SELECT vips_aufgabe.*
                            FROM vips_aufgabe, $g_inAssignment_table
                            WHERE $g_inAssignment_table.$g_assignment_table = '$assignment_id'
                            AND vips_aufgabe = vips_aufgabe.ID");
                $number_of_exercises = $result->getRecordCount();

                // enter user start time the moment he/she first clicks on any exercise
                $result = $vips_db->executeQuery("SELECT * FROM vips_aufgaben_zeit
                            WHERE Kurs              = '$g_outlet->courseID'
                            AND $g_assignment_table = '$assignment_id'
                            AND user_id             = '$g_outlet->userID'");

                if ($result->getRecordCount() == 0) {
                        $ip_address = $_SERVER['REMOTE_ADDR'];
                        $vips_db->executeUpdate("INSERT INTO vips_aufgaben_zeit
                                    SET Kurs                = '$g_outlet->courseID',
                                        $g_assignment_table = '$assignment_id',
                                        user_id             = '$g_outlet->userID',
                                        Beginn              = NOW(),
                                        ip                  = '$ip_address',
                                        vips_aufgabe        = '$exercise_id'");
                }
                
                /******************************************/
                require_once("exercises/$exercise_type.php");

                $o_exercise = new $exercise_type;
                /******************************************/

                $exercise_question      = $o_exercise->getTagsFromXML($exercise_XML, "Question");
                $exercise_question      = $exercise_question[0];
                $exercise_comment       = $o_exercise->getAttributesFromXML($exercise_XML, "CommentField","Visible");
                $exercise_comment       = $exercise_comment[0];
                $exercise_hint          = $o_exercise->getTagsFromXML($exercise_XML, "Hint");
                $exercise_hint          = $exercise_hint[0];
                
                //wikiformat!
                $exercise_question      = formatReady($exercise_question);
                $exercise_hint          = formatReady($exercise_hint);
                $exercise_comment       = formatReady($exercise_comment);

                ###############################################
                #       get answer + correct arrays           #
                ###############################################

                $dom_xml = $o_exercise->getDomXML($exercise_XML);
                $answers = $dom_xml->get_elements_by_tagname("Answer");

                $answer_array   = array();
                $correct_array  = array();
                $answer_default = '';
                $query_default  = '';

                if ($exercise_type == 'sc_exercise' || $exercise_type == 'sco_exercise')
                {
                        foreach ($answers as $key=>$node) {
                                $group = $node->get_attribute("group");
                                if ($group == '') $group = 0;

                                $answer_array[$group][] = formatReady(utf8_decode($node->get_content()));
                                if ($node->get_attribute("Correct") == '1')
                                    $correct_array[$group] = count($answer_array[$group]) - 1;
                        }
                }
                elseif ($exercise_type == 'yn_exercise')
                {
                        foreach ($answers as $key=>$node) {
                                $answer_array[]         = formatReady(utf8_decode($node->get_content()));
                                if ($node->get_attribute("Correct") == '1')
                                        $correct_array[0]       = $key;
                        }

                        $label_yes = $answer_array[0];
                        $label_no  = $answer_array[1];
                }
                elseif ($exercise_type == 'mc_exercise'||
                                $exercise_type == 'mco_exercise'||
                                $exercise_type == 'lt_exercise')
                {
                        foreach ($answers as $node) {
                                $answer_array[]         = formatReady(utf8_decode($node->get_content()));
                                $correct_array[]        = (float) $node->get_attribute("Correct");
                        }
                        if ($exercise_type == 'lt_exercise') {
                                $answer_distance        = $o_exercise->getTagsFromXML($exercise_XML, "AnswerDistance");
                                $answer_distance        = $answer_distance[0];
                        }

                        $label_yes = _('Ja');
                        $label_no  = _('Nein');

                        if ($exercise_type == 'mco_exercise') {
                                $labels = $dom_xml->get_elements_by_tagname("AnswerLabel");
                                foreach ($labels as $node) {
                                    if ($node->get_attribute("Correct") == '1')
                                        $label_yes = utf8_decode($node->get_content());
                                    else
                                        $label_no  = utf8_decode($node->get_content());
                                }
                        }
                }

                elseif ($exercise_type == 'tb_exercise'||
                                $exercise_type == 'pl_exercise'||
                                $exercise_type == 'oc_exercise'||
                                $exercise_type == 'oct_exercise')
                {
                        $node = $answers[0];
                        $answer_array[0]        = utf8_decode($node->get_content());

                        if ($exercise_type == 'tb_exercise') {
                                $answer_default         = $o_exercise->getTagsFromXML($exercise_XML, "AnswerDefault");
                                $answer_default         = $answer_default[0];
                                $answer_distance        = $o_exercise->getTagsFromXML($exercise_XML, "AnswerDistance");
                                $answer_distance        = $answer_distance[0];
                        }
                        if ($exercise_type == 'pl_exercise') {
                                $answer_default         = $o_exercise->getTagsFromXML($exercise_XML, "AnswerDefault");
                                $answer_default         = $answer_default[0];
                                $query_default          = $o_exercise->getTagsFromXML($exercise_XML, "QueryDefault");
                                $query_default          = html_ready($query_default[0]);
                                
                                $answer                 = $answer_array[0];
                                $answer                 = str_replace("m_l_","",$answer);
                                
                                list($answer,$rest)     = split('%T', $answer);
                                
                                $answer_array[0] = $answer;
                        }
                }
                
                elseif ($exercise_type == "rh_exercise")
                {
                        foreach ($answers as $node)
                        {
                                $answer_array[] = formatReady(utf8_decode($node->get_content()));
                                $correct_array[] = $node->get_attribute("Default");
                        }
                        
                        foreach ($dom_xml->get_elements_by_tagname("Default") as $default)
                        {
                                $default_array[] = formatReady(utf8_decode($default->get_content()));
                        }
                }
                
                elseif ($exercise_type == 'zu_exercise') { 
                        //TODO:Zuordnungsaufgabe not yet working in university studip, has to be replaced by code
                        //from studip-impuls
                        /*$exercise_picurl = $o_exercise->getTagsFromXML($exercise_XML, "PictureURL");
                        $exercise_picurl = urldecode($exercise_picurl[0]);
                        $zu_url_for_flash = "";
                        $counter = 0;
                        //$zu_url_for_flash .= "?hintergrund=vips%2Fupload%2F$exercise_picurl";
                        $zu_url_for_flash .= "?hintergrund=vips%2Fupload%2Fbaum.jpg";
                        foreach ($answers as $node) {
//                              $solution = array();
//                              $solution[0] = $node->get_attribute("x1"); 
//                              $solution[1] = $node->get_attribute("y1"); 
//                              $solution[2] = $node->get_attribute("x2"); 
//                              $solution[3] = $node->get_attribute("y2"); 
//                              $correct_array[]        = $solution;
                                
                                $answer_array[]         = urldecode(utf8_decode($node->get_content()));
                                //$zu_url_for_flash .= "&vordergrund_$counter=vips%2Fupload%2F$answer_array[$counter].jpg";
                                $counter++;
                        }
                        //list($zu_pic_width, $zu_pic_height, $type, $attr) = getimagesize("vips/upload/$exercise_picurl");
                        //$zu_url_for_flash .= "&total=$counter";
                        $zu_url_for_flash .= "&vordergrund_0=vips%2Fupload%2Fbild1.jpg&vordergrund_1=vips%2Fupload%2Fbaum.jpg&total=4";
                        */
                }
                
                $pictureURL     = $o_exercise->getTagsFromXML($exercise_XML, "PictureURL");
                $pictureURL     = $pictureURL[0];

                if (is_array($answer_array[0])) {
                    $number_of_answers = implode(',', array_map('count', $answer_array));
                } else {
                    $number_of_answers = count($answer_array);  // stimmt nur begrenzt (see below)
                }
                
                ###################################
                # get user solution if applicable #
                ###################################

                // check if there is already a user's solution OR a group solution

                if ($p_action == 'sheets' && !$single_solver) {
                        // check if there is already a group solution
                        $solver_field = 'Gruppenid';
                } else {
                        // check if the user is in no group but has still submitted an answer
                        $solver_field = 'Userid';
                }

                $sql = "SELECT * FROM $g_solution_table
                                WHERE Aufgabenid      = '".($in_rnd_exercise ? $rnd_exercise_id : $exercise_id)."'
                                 AND  Assignmentid    = '$assignment_id'
                                 AND  $solver_field   = '$solver_id'
                                 AND  isLastSolution  = 't'";
                $result2 = $vips_db->executeQuery($sql);

                if ($result2->next()) {
                        $solution          = $result2->getRow();
                        $solution_XML      = $solution['Loesung'];
                        //vv important: student_array with complete user solution vv
                        $student_array     = $o_exercise->getTagsFromXML($solution_XML, "answer");

                        $student_comment   = $solution['Anmerkung'];
                        $solution_id       = $solution['Loesungsid'];
                        $lecturer_comment  = $solution['Korrektur'];
                        $solved = 1;
                }
                
                if ($exercise_type=="pl_exercise" && $pl_temp_answer) $student_array[0] = $pl_temp_answer;  //PROLOG specific.. If there is a typed-in answer, show that one
                 
                // Reihenfolge exercise: if exercise has been worked on, check if student gave an
                // answer twice
                if ($exercise_type == "rh_exercise" && $solved)
                {
                        if (!$o_exercise->checkSolutionXML($solution_XML))
                        {
                                $message = array('mode' => 'info',
                                                 'message' => _('Vorsicht! Einer einzelnen Vorgabe wurden mehrere Texte zugeordnet.'));
                        }
                }
                
                $max_points = 1;
                $solution_proz = round(($reached_points*100)/$max_points);

                if (!$reached_points) { $reached_points = 0; }
                if (!$solution_proz) { $solution_proz = 0; }


                ##################################################
                #  lecturer: get all course participants         #
                #  (to list for submission of student solution)  #
                ##################################################
                
                $course_participants = array();
                if ($g_outlet->status == 'lecturer' || $g_outlet->status == 'tutor') {

                        $result = $vips_db->executeQuery("SELECT seminar_user.*,
                                                                                auth_user_md5.username,
                                                                                auth_user_md5.Vorname,
                                                                                auth_user_md5.Nachname,
                                                                                auth_user_md5.perms
                                                                        FROM seminar_user,
                                                                                 auth_user_md5
                                                                        WHERE   seminar_user.Seminar_id = '$g_outlet->courseID'
                                                                        AND     seminar_user.user_id    = auth_user_md5.user_id
                                                                        ORDER BY auth_user_md5.Nachname, auth_user_md5.Vorname");

                        while ($result->next()) {
                                $course_participant = $result->getRow();
                                $course_participants[] = $course_participant;
                        }
                }

                #####################################################
                #   set show_solution (to display correct solution  #
                #   in blue in addition to user solution)            #
                #####################################################
                //if a solution has been submitted during a selftest OR lecturer is correcting
                if($korrektur || ($dialog == 'submitted' && $selftest)) {
                        $show_solution = 1;
                }

                ################################
                #     construct templates      #
                ################################

                $t_main_header  = $vipsTemplateFactory->open('exercises/show_exercise_header');
                $t_main         = $vipsTemplateFactory->open('exercises/show_'.$exercise_type);
                $t_main_footer  = $vipsTemplateFactory->open('exercises/show_exercise_footer');

                if ($exercise_type == 'tb_exercise' && !$print) {
                        $t_main_header  = $vipsTemplateFactory->open('exercises/show_exercise_header_tbquer');
                        $t_main         = $vipsTemplateFactory->open('exercises/show_'.$exercise_type.'_tbquer');
                        $t_main_footer  = $vipsTemplateFactory->open('exercises/show_exercise_footer_tbquer');
                }
        
        
        
        ##############################
        #   set template variables   #
        ##############################
        
        /****  set general exercise info variables (show_exercise_header) ****/
        /*********************************************************************/

        /* don't display negative remaining time (unused) */
        if ($remainingMinutes <= 0) $remainingMinutes = 0;
        
        $t_main_header->set_attributes(array(
            'action'                => $p_action,
            'picturepath'           => $g_outlet->picturepath,
            'message'               => $message,
            'solved'                => $solved,
            'selftest'              => $selftest,  //fuer uhranzeige (nur wenn kein selbsttest)
            'show_solution'         => $show_solution,
            
            'time'                  => time(),
            'remainingMinutes'      => $remainingMinutes,
            'studentAbgabe'         => $studentAbgabe,
            'new_solution_id'       => $new_solution_id,
            
            'subaction'             => $show_solution ? 'show_solution'
                                                      : 'submit_exercise',
            
            'assignment_id'         => $assignment_id,
            'exercise_id'           => $exercise_id,
            'exercise_type'         => $exercise_type,
            'exercise_position'     => $exercise_position,
            'exercise_title'        => $exercise_title,
            'exercise_name'         => $exercise_name,
            'exercise_question'     => $exercise_question,
            'exercise_hint'         => $exercise_hint,
            'exercise_general_name' => $o_exercise->g_ExerciseName,
            'exercise_pictureURL'   => $pictureURL,
            
            'solution_id'           => $solution_id,  // can be empty
            'number_of_exercises'   => $number_of_exercises,
            
            'solver_id'             => $solver_id,
            'solved'                => $solved,
            'lecturer_comment'      => $lecturer_comment,
            'reached_points'        => $reached_points,
            'max_points'            => $exercise_max_points,
            'view'                  => $view,
            
            'status'                => $g_outlet->status,
            
            'korrektur'             => $korrektur,
            
            'in_rnd_exercise'       => $in_rnd_exercise,
            'rnd_exercise_id'       => $rnd_exercise_id,
            'rnd_exercise_name'     => $rnd_exercise_name,
            'rnd_exercise_position' => $rnd_exercise_position,
            
            'dialog'                => $dialog));  // internal communication :-)
        
        if ($exercise_type=='tb_exercise' ||
                $exercise_type=='oct_exercise'||
                $exercise_type=='oc_exercise' ||
                $exercise_type=='lt_exercise' ||
                $exercise_type=='pl_exercise')
        {
            $t_main_header->set_attribute('number_of_answers', 1);
            $t_main_header->set_attribute('exercise_number', 1);  // necessary for oc_, oct_, pl_exercises (and in lt and tb it doesn't hurt)
        }
        else {
            $t_main_header->set_attribute('number_of_answers', $number_of_answers);
        }
        
        
        
        /***** set variables for exercise specific template (t_main) ******/
        /******************************************************************/
        
        $t_main->set_attributes(array(
            'picturepath'        => $g_outlet->picturepath,
            'vips_imagepath'     => $vipsPlugin->getPluginPath().'/images/',
            'solved'             => $solved,  // and now the *template* "knows" what to display
            'exercise_name'      => $exercise_name,      // for tb_quer
            'exercise_question'  => $exercise_question,  // for tb_quer
            'exercise_hint'      => $exercise_hint,      // for tb_quer
            'mc_auswertung'      => $mc_auswertung,
            'label_yes'          => $label_yes,          // for yn + mco
            'label_no'           => $label_no,           // for yn + mco
            
            'show_solution'      => $show_solution,
            'korrektur'          => $korrektur,
            'submitted'          => $submitted,
            
            'dialog'             => $dialog,
            
            'answer_array'       => $answer_array,
            'correct_array'      => $correct_array,
            'default_array'      => $default_array,  // for rh
            
            // student answers:
            'student_array'      => $student_array,
            
            'answer_default'     => $answer_default,  // for pl and tb exercise
            'query_default'      => $query_default,  // for pl and oc_exercise
            
            'pl_temp_answer'     => $pl_temp_answer,  // important! (will be displayed if not null)
            'pl_temp_query'      => $pl_temp_query,
            'pl_out'             => $pl_out,          // -"-
            'pl_count'           => $pl_count,        // -"-
            'pl_score'           => $pl_score,
            'pl_validity'        => $pl_validity,
            
//            'zu_url_for_flash'   => $zu_url_for_flash,  // for zu_exercise
//            'zu_pic_width'       => $zu_pic_width,
//            'zu_pic_height'      => $zu_pic_height,
            'exercise_picurl'    => $exercise_picurl,  // for zu_exercise
            
            'student_comment'    => $student_comment,
            'user_solution'      => $user_solution,
            
            'rnd_exercise_empty' => $rnd_exercise_empty));
        
        
        
        /*********** set exercise footer variables (t_main_footer) ************/
        /**********************************************************************/
        
        //$t_main_footer->set_attribute('exercise_solution_proz',       $solution_proz);
        
        $t_main_footer->set_attributes(array(
            'action'              => $p_action,
            'picturepath'         => $g_outlet->picturepath,
            'korrektur'           => $korrektur,
            'status'              => $g_outlet->status,
            'course_participants' => $course_participants,
            'show_solution'       => $show_solution,
            'student_comment'     => $student_comment,  // only set if solved, else empty
            'exercise_comment'    => $exercise_comment,  // true or false
            'assignment_id'       => $assignment_id,
            'exercise_id'         => $exercise_id,
            'exercise_position'   => $exercise_position,
            'exercise_type'       => $exercise_type,
            'dialog'              => $dialog,  // internal communication :-)
            'number_of_exercises' => $number_of_exercises,
            'solved'              => $solved,
            'solver_user_id'      => $solver_user_id,
            'solution_proz'       => $solution_proz));
        
        if ($g_outlet->status == "lecturer")
        {
            // Setting this attribute will result in that a link is shown in the
            // footer which allows to jump back to editing the random exercise.
            // This is ONLY wanted if the LECTURER is looking at the exercise,
            // NOT if a STUDENT is solving the exercise!
            $t_main_footer->set_attribute("in_rnd_exercise", $in_rnd_exercise);
        }
        $t_main_footer->set_attribute("rnd_exercise_id", $rnd_exercise_id);
        $t_main_footer->set_attribute("rnd_exercise_name", $rnd_exercise_name);
        $t_main_footer->set_attribute("rnd_exercise_position", $exercise_position);
        $t_main_footer->set_attribute("rnd_exercise_empty", $rnd_exercise_empty);
        
        if ($print) {
            $t_main_header->set_attribute("print", true);
            $t_main->set_attribute("print", true);
            $t_main_footer->set_attribute("print", true);
            // determine whether to show lecturer's corrections
            //$t_main->set_attribute('show_solution', false);
            $t_main->set_attribute("samplesolution", $samplesolution);
        }
        
        setlocale(LC_NUMERIC, NULL);
        echo $t_main_header->render();
        setlocale(LC_NUMERIC, 'C');

        if (!$dialog || $dialog == 'submitted')  {
            echo $t_main->render();  // normal show exercise
            echo $t_main_footer->render();
        }
    } // endif user has access to exam
}  // end function show_exercise()

/**
 * 
 * Displays all running assignments "work-on ready" for students (view of students when clicking on 
 * "Reiter" Uebungsblatt or Klausur)
 * 
 */
function list_assignments_stud($p_action) {
        global $vips_db,
               $g_outlet,
               $g_title_p,
               $g_inAssignment_table,
               $g_assignment_table,
               $g_solution_table,
               $vipsTemplateFactory;

        // messages for no access, no time, interrupted (array)
        $dialog = getExternalParam('dialog');
        $message = getExternalParam('message');

        $delete_assignment_id   = getExternalParam('delete_assignment_id');

        $assignments_select = array();
        $assignments        = array();

        $t_list = $vipsTemplateFactory->open('list_assignments_stud');
        $t_list->set_attribute('picturepath', $g_outlet->picturepath);
        $t_list->set_attribute('action', $p_action);
        $t_list->set_attribute('status', $g_outlet->status);
        $t_list->set_attribute('assignment_id', $delete_assignment_id);

        if ($g_outlet->status == 'lecturer' || $g_outlet->status == 'tutor') {
                if ($assignment_id = getExternalParam('assignment_id')) {       //lecturer wants to display selected assignment for himself
                        $result = $vips_db->executeQuery("SELECT * FROM $g_assignment_table WHERE ID = '$assignment_id'");
                        $result->next();
                        $assignment = $result->getRow();
                        $assignments_select[] = $assignment;
                        $t_list->set_attribute('assignment_id', $assignment_id);
                        
                        $now = date("Y-m-d H:i:s");
                        if ($now > $assignment["Ende"]) {
                                $running_status = "expired";
                        } else if ($now < $assignment["Beginn"]) {
                                $running_status = "pending";
                        } else {
                                $running_status = "running";
                        }
                        
                        // fetch information on users which are working on this assignment
                        $query = "SELECT DISTINCT
                                        `vips_aufgaben_zeit`.`user_id`,
                                        `vips_aufgaben_zeit`.`Beginn`,
                                        `vips_aufgaben_zeit`.`ip`,
                                        `auth_user_md5`.`username`,
                                        ".$GLOBALS['_fullname_sql']['full']." AS fullname
                                FROM
                                        `vips_aufgaben_zeit`,
                                        `auth_user_md5`
                                LEFT JOIN
                                        user_info USING (user_id)
                                WHERE `vips_aufgaben_zeit`.`Kurs` = '".$g_outlet->courseID."'
                                        AND `vips_aufgaben_zeit`.`vips_klausur` = '$assignment_id'
                                        AND `vips_aufgaben_zeit`.`user_id` = `auth_user_md5`.`user_id`
                                GROUP BY `vips_aufgaben_zeit`.`user_id`
                                ORDER BY
                                        `auth_user_md5`.`Nachname`,
                                        `auth_user_md5`.`Vorname`
                                ";
                        $result = $vips_db->executeQuery($query);

                        while ($result->next())
                        {
                                $remaining_time = clip_remaining_time($result->get('Beginn'), $assignment['Dauer'], $assignment['Ende']);
                                $running_exam_info[] = array
                                        ( "username"       => $result->get("username")
                                        , "name"           => $result->get("fullname")
                                        , "ip"             => $result->get("ip")." (".gethostbyaddr($result->get("ip")).")"
                                        , "start_time"     => date("H:i:s", strtotime($result->get("Beginn")))
                                        , "remaining_time" => $remaining_time);
                        }
                }
        }
        else { //get all running exercises for student
                
                $result3 = $vips_db->executeQuery("SELECT * FROM $g_assignment_table
                                        WHERE Kurs='$g_outlet->courseID'
                                        ORDER BY Beginn, Position");
                while ($result3->next()) {
                        $assignment = $result3->getRow();
                        $m_start_timestamp = strtotime($assignment['Beginn']);
                        $m_end_timestamp   = strtotime($assignment['Ende']);
                        if (time() <= $m_end_timestamp) {
                                if (time() < $m_start_timestamp) {
                                        $assignment['hidden'] = TRUE;
                                }
                                $assignments_select[] = $assignment;
                        }
                }
        }

        foreach ($assignments_select as $assignment) {
                $assignment_id = $assignment['ID'];

                $exercises = array();
                $sql =  "SELECT vips_aufgabe.*,
                                                $g_inAssignment_table.Position,
                                                $g_inAssignment_table.Punkte,
                                                vips_aufgaben_typen.Typenname
                                        FROM  vips_aufgabe,
                                                  $g_inAssignment_table,
                                                  $g_assignment_table,
                                                  vips_aufgaben_typen
                                        WHERE $g_inAssignment_table.$g_assignment_table = $g_assignment_table.ID
                                        AND   $g_assignment_table.ID            = '$assignment_id'
                                        AND   vips_aufgaben_typen.URI           = vips_aufgabe.URI
                                        AND   vips_aufgabe.ID                           = $g_inAssignment_table.vips_aufgabe
                                        ORDER BY $g_inAssignment_table.Position
                                                ";
                $result2 = $vips_db->executeQuery($sql);

                $assignment['ippermitted'] = true;
                if ($g_outlet->status == 'student' && $p_action == 'exams') {
                        $assignment['ippermitted'] = ip_permitted($assignment_id);
                }


                #########################
                # get old user solution #
                #########################

                //get group id if applicable
                ////////////////////////////////new///////////////////////////////////////
                $result4 = $vips_db->executeQuery(
                                "SELECT vips_inGruppe.* FROM
                                                vips_inGruppe,
                                                vips_gruppe
                                        WHERE user_id = '$g_outlet->userID'
                                        AND  vips_gruppe.Gruppenid      = vips_inGruppe.group_id
                                        AND  vips_gruppe.Kursid         = '$g_outlet->courseID'
                                        AND  vips_inGruppe.end IS NULL");
                $group_id = 0;
                if ($result4->next()) {
                        $group = $result4->getRow();
                        $group_id = $group['group_id'];
                }
                
                
                while ($result2->next()) {
                        $exercise = $result2->getRow();
                        $exercise['Punkte'] = $result2->getFloat('Punkte');
                        $exercise_id    = $exercise['ID'];
                        $user_id                = $g_outlet->userID;
                        
                        if ($p_action == 'sheets' && $group_id != 0) {
                                $sql = "SELECT * FROM $g_solution_table
                                                WHERE Aufgabenid = '$exercise_id'
                                                AND Assignmentid = '$assignment_id'
                                                AND (Userid='$user_id' OR Gruppenid = '$group_id')
                                                AND isLastSolution = 't'";
                        } else {
                                $sql = "SELECT * FROM $g_solution_table
                                                WHERE Aufgabenid = '$exercise_id'
                                                AND Assignmentid = '$assignment_id'
                                                AND Userid='$user_id'
                                                AND isLastSolution = 't'";
                        }
                        $result3 = $vips_db->executeQuery($sql);

                        if ($result3->next()) {// means exercise is already answered
                                $exercise['solved'] = 1;
                        }
                        $exercises[] = $exercise;
                }
                
                ///////////////////////////////////////////////////////////////////////
                $assignment['exercises'] = $exercises;
                if ($p_action == 'exams') {
                        $assignment['remainingMinutes'] = getRemainingMinutes($assignment_id);

                        /* don't display negative remaining time */
                        if ($assignment['remainingMinutes'] <= 0)
                            $assignment['remainingMinutes'] = 0;
                }
                $assignments[] = $assignment;
        }

        $t_list->set_attribute('message', $message);
        $t_list->set_attribute('assignments', $assignments);
        $t_list->set_attribute('dialog', $dialog);
        $t_list->set_attribute('ip_address', $_SERVER['REMOTE_ADDR']);
        $t_list->set_attribute('hostname', gethostbyaddr($_SERVER['REMOTE_ADDR']));
        $t_list->set_attribute('title', "Laufende $g_title_p");
        $t_list->set_attribute("running_status", $running_status);
        $t_list->set_attribute("running_exam_info", $running_exam_info);  // only set in running exams

        setlocale(LC_NUMERIC, NULL);
        echo $t_list->render();
        setlocale(LC_NUMERIC, 'C');
}

#####################################
#####################################
#                                                                       #
#               Lecturer Methods                        #
#                                                                       #
#####################################
#####################################


/**
 * SHEETS/EXAMS (blocks only for sheets)
 * 
 * Stores assignment options such as block membership or selftest
 */

function assignment_options($p_action) {
        global $vips_db,
                   $g_outlet,
                   $g_inAssignment_table,
                   $g_assignment_table;

        $number_of_assignments = getExternalParam("number_of_assignments");

        for ($i = 0; $i < $number_of_assignments; ++$i) {

                $assignment_id  = getExternalParam("assignment_no_$i");
                $block_id               = getExternalParam("assign_to_block_$i");
                $selftest               = getExternalParam("selftest_for_$assignment_id");

                /**** selftest *****************************************/

                if ($selftest) $selftest = 1;
                $vips_db->executeUpdate("UPDATE $g_assignment_table
                                        SET     Selbsttest      = '$selftest'
                                        WHERE   ID                      = '$assignment_id'");


                /**** Bloecke *************************************************/

                if              ($block_id == "0") {}
                elseif  ($block_id == "-1") {                           //remove membership to block
                        $vips_db->executeUpdate("DELETE FROM vips_inBlock
                                                WHERE assignment_id = '$assignment_id'
                                                ");
                }
                else {                                                                          //assign to block
                        $result = $vips_db->executeQuery("SELECT * FROM vips_inBlock
                                                WHERE   assignment_id = '$assignment_id'
                                                ");

                        if ($result->getRecordCount() == 0) {
                                $vips_db->executeUpdate("INSERT INTO vips_inBlock
                                                                        SET block_id            = '$block_id',
                                                                                assignment_id   = '$assignment_id'
                                                                                ");
                        }
                        else {
                                $vips_db->executeUpdate("UPDATE vips_inBlock
                                                SET     block_id                = '$block_id'
                                                WHERE   assignment_id   = '$assignment_id'
                                                ");
                        }
                }
        }
        list_assignments($p_action);  //normal view
}



/**
 * SHEETS
 * 
 * Adds a block name to the database so sheets can be assigned to this block
 * 
 */
 
function add_block($p_action) {
        global $vips_db,
                   $g_outlet,
                   $g_inAssignment_table,
                   $g_assignment_table;

        $new_block_name = getExternalParam('new_block_name');

        //TODO auf doppelte Namen ueberpruefen!!!
        
        $vips_db->executeUpdate("INSERT INTO vips_block
                                SET
                                        Blockname               = '$new_block_name',
                                        Kurs                    = '$g_outlet->courseID'
                                ");
        
        list_assignments($p_action);
}

/**
 * EXAMS/SHEETS
 * 
 * If an assignment hasn't started yet this function sets the start time to NOW 
 * so that it's running
 * 
 */

function start_assignment($p_action) {
        global $vips_db,
                   $g_outlet,
                   $g_inAssignment_table,
                   $g_assignment_table;
        
        $assignment_id = getExternalParam('assignment_id');
        $date          = date("Y-m-d H:i:s");
        
        // set new Startzeitpunkt in database
        $vips_db->executeUpdate("UPDATE $g_assignment_table
                    SET Beginn = '$date', istGesperrt = 'n'
                    WHERE ID = '$assignment_id'");
        
        // delete Startzeitpunkte for exam from database
        $vips_db->executeUpdate("DELETE FROM vips_aufgaben_zeit
                    WHERE Kurs = '".$g_outlet->courseID."'
                    AND $g_assignment_table = '$assignment_id'");

        list_assignments($p_action);
}


/**
 * EXAMS/SHEETS
 * 
 * Stops/continues an assignment (no change of start/end time but temporary closure)
 * 
 */
function stopgo_assignment ($p_action) {
        global $vips_db,
                   $g_outlet,
                   $g_inAssignment_table,
                   $g_assignment_table;

        $assignment_id                  = getExternalParam('assignment_id');

        $sql = "UPDATE $g_assignment_table
                        SET istGesperrt = (CASE WHEN istGesperrt = 'y' THEN 'n' ELSE 'y' END)
                        WHERE ID = '$assignment_id'";
        $vips_db->executeUpdate($sql);

        list_assignments($p_action);
}


/**
 * EXAMS/SHEETS
 * 
 * TODO: Assignment don't have a position right now (they are listed according to their start date),
 * but it might be nice to have a order which the lecturer can manipulate
 */
function move_assignment($p_action) {
        /*????*/
}

/**
 * EXAMS/SHEETS
 * 
 * Deletes an assignment from the course (and block if applicable). All exercises are retained.
 */
function delete_assignment($p_action) {
        global $vips_db,
               $g_outlet,
               $g_inAssignment_table,
               $g_assignment_table;

        $assignment_id  = getExternalParam('assignment_id');
        $position       = getExternalParam('assignment_position');
        
        if (getExternalParam('cancel') || getExternalParam('cancel_x') != '')
        {
                list_assignments($p_action);
        }
        else if (getExternalParam('delete_assignment_confirmed') || getExternalParam('delete_assignment_confirmed_x') != '')
        {
                $vips_db->executeUpdate("DELETE FROM $g_inAssignment_table
                                        WHERE $g_assignment_table = '$assignment_id'");

                $vips_db->executeUpdate("DELETE FROM vips_inBlock
                                        WHERE assignment_id = '$assignment_id'");
                //Position: ???
                //$vips_db->executeUpdate("UPDATE " . $g_sheetType . " SET Position=Position-1 WHERE Kurs='".$m_range_id."' AND Position>'" . $p_exercise_sheet_position."'");

                $vips_db->executeUpdate("DELETE FROM $g_assignment_table WHERE ID = '$assignment_id'");
                $message['mode']        = 'msg';
                $message['message']     = 'Das Aufgabenblatt wurde gelscht.';
                $_POST['message']       = $message;
                list_assignments($p_action);
        }
        else
        {
                $assignment = get_assignment_data($assignment_id);
                $_POST['delete_assignment_id']   = $assignment_id;
                $_POST['delete_assignment_name'] = $assignment['Name'];
                $_POST['dialog'] = 'delete_assignment_confirm';
                list_assignments($p_action);
        }
}



/**
 * SHEETS/EXAMS
 * 
 * Takes an exercise off an assignment. If $reallydelete is set, the exercise is also completely
 * removed from the database.
 */
function delete_exercise($p_action) {
        global $vips_db,
                   $g_outlet,
                   $g_inAssignment_table,
                   $g_assignment_table;

        $exercise_id         = getExternalParam('exercise_id');
        $assignment_id       = getExternalParam('assignment_id');
        $position            = getExternalParam('exercise_position');
        $number_of_exercises = getExternalParam('number_of_exercises');
        $reallydelete        = getExternalParam('reallydelete');

        // TODO replace this with proper access validation and move output to template
        // The access to this function is forbidden if the assignment has expired! //

        /* disabled for now, see http://plugins.studip.de/trac/ticket/32
        $query = "SELECT `Ende` FROM $g_assignment_table WHERE `ID` = $assignment_id";
        $result = $vips_db->executeQuery($query);
        $result->next();
        if (date("Y-m-d H:i:s") > $result->get("Ende"))
        {
                echo _("Das bungsblatt ist beendet. Aufgaben in abgelaufenen bungsblttern drfen nicht mehr gelscht werden!");
                return;
        }
        */

        // if exercise is in a rnd_exercise: delete it from rnd_exercise and leave
        // assignment sheets and `vips_aufgabe` untouched
        if (getExternalParam("in_rnd_exercise"))
        {
                require_once("exercises/rnd_exercise.php");
                $rnd0_exercise = new rnd_exercise();
                
                $rnd_exercise_id = getExternalParam("rnd_exercise_id");
                
                // fetch rnd_exercise XML
                $query = "SELECT `Aufgabe`
                        FROM `vips_aufgabe`
                        WHERE `ID` = '".$rnd_exercise_id."'
                        ";
                $result = $vips_db->executeQuery($query);
                $result->next();
                $xml = $result->get("Aufgabe");
                
                $xml = $rnd0_exercise->delete_subexercise($xml, $exercise_id);
                
                $query = "UPDATE `vips_aufgabe`
                        SET `Aufgabe` = '".$xml."'
                        WHERE `ID` = '".$rnd_exercise_id."'
                        ";
                $vips_db->executeUpdate($query);

                $_POST["assignment_id"]     = $assignment_id;
                $_POST["exercise_id"]       = $rnd_exercise_id;
                $_POST["exercise_type"]     = "rnd_exercise";
                $_POST['exercise_position'] = getExternalParam('rnd_exercise_position');
                // jump to editing the rnd_exercise itself, not any subexercise
                unset($_GET['in_rnd_exercise']);
                edit_exercise($p_action);
                return;
        }

        // remove exercise
        $query1 = "DELETE FROM $g_inAssignment_table
                    WHERE vips_aufgabe = '$exercise_id'
                    AND $g_assignment_table = '$assignment_id'
                    AND Position = '$position'";

        // renumber following exercises
        $query2 = "UPDATE $g_inAssignment_table
                    SET Position = Position - 1
                    WHERE $g_assignment_table = '$assignment_id'
                    AND Position > '$position'";

        $affected_rows = $vips_db->executeUpdate($query1);

        if ($affected_rows) {
            $vips_db->executeUpdate($query2);
        }

        if ($reallydelete) {
            $query3 = "DELETE FROM vips_aufgabe WHERE ID = '$exercise_id'";
            $vips_db->executeUpdate($query3);
        }

        $_POST['assignment_id'] = $assignment_id;
        $_POST['number_of_exercises'] = $number_of_exercises-1;
        edit_assignment($p_action);
}

/**
 * List all exercises in the given assignment (AJAX).
 */
function list_exercises_ajax ($action, $assignment_id)
{
    global $vips_db, $g_outlet, $vipsTemplateFactory,
           $g_assignment_table, $g_inAssignment_table;

    #############################################
    # get existing exercises in this assignment #
    #############################################

    $query = "SELECT vips_aufgabe.*,
                  vips_aufgaben_typen.Typenname,
                  $g_inAssignment_table.Position,
                  $g_inAssignment_table.Punkte,
                  $g_inAssignment_table.Bewertet
              FROM vips_aufgabe,
                  vips_aufgaben_typen,
                  $g_inAssignment_table
              WHERE $g_assignment_table = '$assignment_id'
                  AND vips_aufgabe = vips_aufgabe.ID
                  AND vips_aufgaben_typen.URI = vips_aufgabe.URI
              ORDER BY $g_inAssignment_table.Position";

    $result = $vips_db->executeQuery($query);
    $exercises = array();

    while ($result->next()) {
        $entry = $result->getRow();
        $entry['Punkte'] = $result->getFloat('Punkte');
        $exercises[] = $entry;
    }

    $template = $vipsTemplateFactory->open('list_exercises');

    $template->set_attributes(array(
        'picturepath'            => $g_outlet->picturepath,
        'action'                 => $action,
        'assignment_id'          => $assignment_id,
        'exercises'              => $exercises
    ));

    // this hack is necessary to disable the standard Stud.IP layout
    ob_end_clean();
    header('Content-Type: text/html; charset=ISO-8859-1');

    setlocale(LC_NUMERIC, NULL);
    echo $template->render();
    setlocale(LC_NUMERIC, 'C');

    ob_start('discard_buffer');
}

/**
 * Takes an exercise off an assignment. If $reallydelete is set, the exercise is also completely
 * removed from the database (AJAX).
 */
function delete_exercise_ajax ($action)
{
    global $vips_db, $g_inAssignment_table, $g_assignment_table;

    $exercise_id         = getExternalParam('exercise_id');
    $assignment_id       = getExternalParam('assignment_id');
    $position            = getExternalParam('exercise_position');
    $reallydelete        = getExternalParam('reallydelete');

    // remove exercise
    $query1 = "DELETE FROM $g_inAssignment_table
                WHERE vips_aufgabe = '$exercise_id'
                AND $g_assignment_table = '$assignment_id'
                AND Position = '$position'";

    // renumber following exercises
    $query2 = "UPDATE $g_inAssignment_table
                SET Position = Position - 1
                WHERE $g_assignment_table = '$assignment_id'
                AND Position > '$position'";

    $affected_rows = $vips_db->executeUpdate($query1);

    if ($affected_rows) {
        $vips_db->executeUpdate($query2);
    }

    if ($reallydelete) {
        $query3 = "DELETE FROM vips_aufgabe WHERE ID = '$exercise_id'";
        $vips_db->executeUpdate($query3);
    }

    list_exercises_ajax($action, $assignment_id);
}

/**
 * SHEETS/EXAMS
 * 
 * Changes position of an exercise within an assignment
 * 
 */
function move_exercise($p_action) {
        global $vips_db,
                   $g_outlet,
                   $g_inAssignment_table,
                   $g_assignment_table;
        $direction                              = getExternalParam('direction');
        $exercise_id                    = getExternalParam('exercise_id');
        $assignment_id                  = getExternalParam('assignment_id');
        $position                               = getExternalParam('exercise_position');
        $number_of_exercises    = getExternalParam('number_of_exercises');

        // TODO replace this with proper access validation and move output to template
        // The access to this function is forbidden if the assignment has expired! //

        /* disabled for now, see http://plugins.studip.de/trac/ticket/32
        $query = "SELECT `Ende` FROM $g_assignment_table WHERE `ID` = $assignment_id";
        $result = $vips_db->executeQuery($query);
        $result->next();
        if (date("Y-m-d H:i:s") > $result->get("Ende"))
        {
                echo _("Das bungsblatt ist beendet. Die Reihenfolge der Aufgaben in abgelaufenen bungsblttern darf nicht mehr verndert werden!");
                return;
        }
        */

        if ($direction == 'down')
            $position_target = $position - 1;
        else
            $position_target = $position + 1;

        
        // move selected
        $query1 = "UPDATE $g_inAssignment_table
                    SET Position = '$position_target'
                    WHERE vips_aufgabe = '$exercise_id'
                    AND Position = '$position'
                    AND $g_assignment_table = $assignment_id";
        
        // move other exercises
        $query2 = "UPDATE $g_inAssignment_table
                    SET Position = '$position'
                    WHERE vips_aufgabe != '$exercise_id'
                    AND Position = '$position_target'
                    AND $g_assignment_table = $assignment_id";

        $affected_rows = $vips_db->executeUpdate($query1);

        if ($affected_rows) {
            $vips_db->executeUpdate($query2);
        }

        $_POST['assignment_id'] = $assignment_id;
        $_POST['number_of_exercises'] = $number_of_exercises;
        edit_assignment($p_action);
}

/**
 * Reorder exercise positions within an assignment (AJAX).
 */
function move_exercise_ajax ($action)
{
    global $vips_db, $g_inAssignment_table;

    $assignment_id = getExternalParam('assignment_id');
    $list          = getExternalParam('list');

    /* renumber all exercises in current assignment */
    for ($i = 0; $i < count($list); ++$i)
    {
        $sql = "UPDATE $g_inAssignment_table SET Position = ".($i+1)." WHERE vips_aufgabe = '".$list[$i]."'";
        $vips_db->executeUpdate($sql);
    }

    list_exercises_ajax($action, $assignment_id);
}

/**
 * Return list of semesters with assignments accessible to teachers
 * in the current course. This includes assignments from other courses.
 */
function get_accessible_semesters ()
{
    global $vips_db, $g_outlet, $g_assignment_table;

    $sql = "SELECT DISTINCT
                semester_data.name,
                semester_data.beginn
            FROM
                semester_data,
                $g_assignment_table,
                seminare,
                seminar_user,
                seminar_user AS lecturer
            WHERE
                $g_assignment_table.Kurs = seminare.Seminar_id AND
                seminare.start_time = semester_data.beginn AND
                seminar_user.Seminar_id = seminare.Seminar_id AND
                seminar_user.status = 'dozent' AND
                seminar_user.user_id = lecturer.user_id AND
                lecturer.Seminar_id = '$g_outlet->courseID' AND
                lecturer.status = 'dozent'
            ORDER BY
                beginn DESC";

    $result = $vips_db->executeQuery($sql);
    while ($result->next())
    {
        $semesters[] = array('name'   => $result->getString('name'),
                             'beginn' => $result->getInt('beginn'));
    }

    return $semesters;
}

/**
 * Return data of the semester starting at the given time.
 */
function get_semester_data ($start_time)
{
    global $vips_db;

    $sql = "SELECT name, beginn FROM semester_data WHERE beginn = '$start_time'";

    $result = $vips_db->executeQuery($sql);
    $result->next();
    $semester = array('name'   => $result->getString('name'),
                      'beginn' => $result->getInt('beginn'));

    return $semester;
}

/**
 * Return list of courses with assignments accessible to teachers
 * in the current course. This includes assignments from other courses.
 */
function get_accessible_courses ($start_time)
{
    global $vips_db, $g_outlet, $g_assignment_table;

    $sql = "SELECT DISTINCT
                $g_assignment_table.*
            FROM
                $g_assignment_table,
                seminare,
                seminar_user,
                seminar_user AS lecturer
            WHERE
                $g_assignment_table.Kurs = seminare.Seminar_id AND
                seminare.start_time = '$start_time' AND
                seminar_user.Seminar_id = seminare.Seminar_id AND
                seminar_user.status = 'dozent' AND
                seminar_user.user_id = lecturer.user_id AND
                lecturer.Seminar_id = '$g_outlet->courseID' AND
                lecturer.status = 'dozent'
            ORDER BY
                Beginn, Position";

    $result = $vips_db->executeQuery($sql);
    while ($result->next())
    {
        $course = $result->getString('Kurs');
        $header = html_entity_decode(getHeaderLine($course), ENT_QUOTES);
        $courses[$course]['name'] = $header;
        $courses[$course]['tests'][] = $result->getRow();
    }

    return $courses;
}

/**
 * Return data of the specified assignment.
 */
function get_assignment_data ($test_id)
{
    global $vips_db, $g_assignment_table;

    $sql = "SELECT * FROM $g_assignment_table WHERE ID = '$test_id'";

    $result = $vips_db->executeQuery($sql);
    $result->next();
    $test = $result->getRow();

    return $test;
}

/**
 * Return list of exercises in the specified assignment.
 */
function get_exercises ($test_id)
{
    global $vips_db, $g_assignment_table, $g_inAssignment_table;

    $sql = "SELECT
                vips_aufgabe.*,
                $g_inAssignment_table.Position
            FROM
                vips_aufgabe,
                $g_inAssignment_table
            WHERE
                vips_aufgabe.ID = $g_inAssignment_table.vips_aufgabe AND
                $g_inAssignment_table.$g_assignment_table = '$test_id'
            ORDER BY
                Position";

    $result = $vips_db->executeQuery($sql);
    while ($result->next())
    {
        $exercises[] = $result->getRow();
    }

    return $exercises;
}

/**
 * Return list of exercises matching the search string from all semesters
 * with assignments accessible to teachers in the current course. This
 * includes assignments from other courses.
 */
function get_matching_exercises ($search)
{
    global $vips_db, $g_outlet, $g_assignment_table, $g_inAssignment_table;

    $sql = "SELECT DISTINCT
                semester_data.name AS sem_name,
                semester_data.beginn,
                seminare.Name AS course_name,
                $g_assignment_table.ID as test_id,
                $g_assignment_table.Name as test_name,
                $g_inAssignment_table.Position,
                vips_aufgabe.*
            FROM
                semester_data,
                $g_assignment_table,
                seminare,
                seminar_user,
                seminar_user AS lecturer,
                $g_inAssignment_table,
                vips_aufgabe
            WHERE
                $g_assignment_table.Kurs = seminare.Seminar_id AND
                seminare.start_time = semester_data.beginn AND
                seminar_user.Seminar_id = seminare.Seminar_id AND
                seminar_user.status = 'dozent' AND
                seminar_user.user_id = lecturer.user_id AND
                lecturer.Seminar_id = '$g_outlet->courseID' AND
                lecturer.status = 'dozent' AND
                $g_inAssignment_table.$g_assignment_table = $g_assignment_table.ID AND
                vips_aufgabe.ID = $g_inAssignment_table.vips_aufgabe AND
                vips_aufgabe.aufgabe LIKE '%".htmlspecialchars($search)."%'
            ORDER BY
                beginn DESC, course_name, test_name, Name";

    $result = $vips_db->executeQuery($sql);
    while ($result->next())
    {
        $exercise = $result->getRow();
        // NOTE: strip_tags($exercise['Aufgabe']) removes CDATA sections
        $exercise_xml = domxml_open_mem(utf8_encode($exercise['Aufgabe']));
        $exercise_text = utf8_decode($exercise_xml->get_content());

        // TODO use stripos on PHP 5
        if (strpos(strtolower($exercise_text), strtolower($search)) !== false) {
            $exercises[] = $exercise;
        }
    }

    return $exercises;
}

/**
 * Pick a semester for copying an exercise (AJAX).
 */
function pick_semester_ajax ($action)
{
    global $g_outlet, $vipsPlugin, $vipsTemplateFactory;

    $assignment_id = getExternalParam('assignment_id');
    $start_time    = getExternalParam('start_time');
    $search        = getExternalParam('search');
    $close         = getExternalParam('close');

    $template = $vipsTemplateFactory->open('pick_semester');
    $template->set_attributes(array(
        'picturepath'           => $g_outlet->picturepath,
        'vips_imagepath'        => $vipsPlugin->getPluginPath().'/images/',
        'action'                => $action,
        'assignment_id'         => $assignment_id,
        'search'                => $search
    ));

    // get semester, course, assignment and exercise data for copying.
    // available are all assignments that all lecturers in this course
    // have access to (including other courses)

    $template->set_attribute('semester', get_semester_data($start_time));

    if (empty($close))
    {
        $template->set_attribute('start_time', $start_time);
        $template->set_attribute('courses', get_accessible_courses($start_time));
    }

    // this hack is necessary to disable the standard Stud.IP layout
    ob_end_clean();
    header('Content-Type: text/html; charset=ISO-8859-1');

    echo $template->render();

    ob_start('discard_buffer');
}

/**
 * Pick a test for copying an exercise (AJAX).
 */
function pick_test_ajax ($action)
{
    global $g_outlet, $vipsPlugin, $vipsTemplateFactory;

    $assignment_id = getExternalParam('assignment_id');
    $start_time    = getExternalParam('start_time');
    $test_id       = getExternalParam('test_id');
    $search        = getExternalParam('search');
    $close         = getExternalParam('close');

    $template = $vipsTemplateFactory->open('pick_test');
    $template->set_attributes(array(
        'picturepath'           => $g_outlet->picturepath,
        'vips_imagepath'        => $vipsPlugin->getPluginPath().'/images/',
        'action'                => $action,
        'assignment_id'         => $assignment_id,
        'start_time'            => $start_time,
        'search'                => $search
    ));

    // get semester, course, assignment and exercise data for copying.
    // available are all assignments that all lecturers in this course
    // have access to (including other courses)

    $template->set_attribute('semester', get_semester_data($start_time));
    $template->set_attribute('test', get_assignment_data($test_id));

    if (empty($close))
    {
        $template->set_attribute('test_id', $test_id);
        $template->set_attribute('all_exercises', get_exercises($test_id));
    }

    // this hack is necessary to disable the standard Stud.IP layout
    ob_end_clean();
    header('Content-Type: text/html; charset=ISO-8859-1');

    echo $template->render();

    ob_start('discard_buffer');
}

/**
 * SHEETS/EXAMS
 * 
 * Displays the form for editing an exercise.
 * 
 * Is called when editing an existing exercise or creating a new exercise.
 * 
 * 
 */
function edit_exercise($p_action, $p_copy = false) {
    global $vips_db,
        $g_outlet,
        $g_inAssignment_table,
        $g_assignment_table,
        $vipsTemplateFactory;
    
    $assignment_id         = getExternalParam('assignment_id');
    $exercise_id           = getExternalParam('exercise_id');
    $exercise_position     = getExternalParam('exercise_position');
    $number_of_answers     = getExternalParam('number_of_answers');
    
    $dialog                = $_POST['dialog'];   //array!
    
    $in_rnd_exercise       = getExternalParam("in_rnd_exercise");  // Flag. Determines whether the newly generated exercise will be linked to the assignment or to the calling random exercise.
    $rnd_exercise_id       = getExternalParam("rnd_exercise_id");
    $rnd_exercise_name     = getExternalParam("rnd_exercise_name");
    $rnd_exercise_position = getExternalParam("rnd_exercise_position");
    
    // TODO replace this with proper access validation and move output to template
    // The access to this function is forbidden if the assignment has expired! //
    
    /* disabled for now, see http://plugins.studip.de/trac/ticket/32
    $query = "SELECT `Ende` FROM $g_assignment_table WHERE `ID` = $assignment_id";
    $result = $vips_db->executeQuery($query);
    $result->next();
    if (date("Y-m-d H:i:s") > $result->get("Ende"))
    {
        echo _("Das bungsblatt ist beendet. Aufgaben in abgelaufenen bungsblttern drfen nicht mehr bearbeitet werden!");
        return;
    }
    */
    
    // get mc_auswertung
    $result = $vips_db->executeQuery("SELECT mc_auswertung FROM $g_assignment_table WHERE ID = '$assignment_id'");
    $result->next();
    $assignment = $result->getRow();
    $mc_auswertung = $assignment['mc_auswertung'];

    // fetch all exercises in the assignment
    $result = $vips_db->executeQuery(
                    "SELECT vips_aufgabe.*
                      FROM  vips_aufgabe, $g_inAssignment_table
                      WHERE $g_inAssignment_table.$g_assignment_table = '$assignment_id'
                      AND vips_aufgabe = vips_aufgabe.ID
                      ");
    $number_of_exercises = $result->getRecordCount();

    // fetch all existing exercise types
    $sql = "SELECT `Typenname`, `URI` FROM `vips_aufgaben_typen`";
    $result = $vips_db->executeQuery($sql);
    while ($result->next())
    {
        $uri = $result->get('URI');
        $name = $result->get('Typenname');
        $all_exercise_types[$uri] = $name;
    }
    // The array $all_exercise_types will be passed to the footer template
    // later.
    
    if (getExternalParam('new_exercise'))  // then I'm coming from create_new_exercise
    {
        $exercise_type      = getExternalParam('new_exercise_type');
        $exercise_typenname = $all_exercise_types[$exercise_type];
        
        /*******  default values for exercises *********/
        $number_of_answers = 5;

        $label_yes = _('Ja');
        $label_no  = _('Nein');

        $exercise_id = '';  // necessary
    }
    else
    {
        // edit already existing exercise

        if ($p_copy || $in_rnd_exercise)  // edit copy or subexercise
        {
            // fetch exercise info using exercise id
            $query = "SELECT
                    vips_aufgabe.*,
                    vips_aufgaben_typen.Typenname
                FROM
                    vips_aufgabe,
                    vips_aufgaben_typen
                WHERE
                    vips_aufgabe.ID = $exercise_id
                    AND vips_aufgaben_typen.URI = vips_aufgabe.URI
                ";
        }
        else
        {
            // assure valid exercise position (to prevent SQL failure)
            $exercise_position = min(max(1, $exercise_position), $number_of_exercises);
            
            // fetch exercise info using assignment and position
            $query = "SELECT
                    vips_aufgabe.*,
                    vips_aufgaben_typen.Typenname
                FROM
                    vips_aufgabe,
                    $g_inAssignment_table,
                    vips_aufgaben_typen
                WHERE
                    $g_inAssignment_table.$g_assignment_table = $assignment_id
                    AND $g_inAssignment_table.Position = $exercise_position
                    AND $g_inAssignment_table.vips_aufgabe = vips_aufgabe.ID
                    AND vips_aufgaben_typen.URI = vips_aufgabe.URI
                ";
        }

        $result = $vips_db->executeQuery($query);
        $result->next();
        $entry = $result->getRow();
        
        $exercise_id        = $result->getInt('ID');
        $exercise_name      = html_ready($entry['Name']);
        $xml                = $entry['Aufgabe'];
        $exercise_type      = $entry['URI'];
        $exercise_typenname = $entry['Typenname'];


                ///////////////////////////////////////////

                require_once("exercises/$exercise_type.php");

                $o_exercise = new $exercise_type();

                ///////////////////////////////////////////

                $exercise_question      = $o_exercise->getTagsFromXML($xml, "Question");
                $exercise_question      = $exercise_question[0];
                $exercise_comment       = $o_exercise->getAttributesFromXML($xml, "CommentField","Visible");
                $exercise_comment       = $exercise_comment[0];
                $exercise_hint          = $o_exercise->getTagsFromXML($xml, "Hint");
                $exercise_hint          = $exercise_hint[0];

                // format for output
                $exercise_question      = html_ready($exercise_question);
                $exercise_hint          = html_ready($exercise_hint);
                $exercise_comment       = html_ready($exercise_comment);

                $dom_xml = $o_exercise->getDomXML($xml);
                $answers = $dom_xml->get_elements_by_tagname("Answer");
                
                $answer_array   = array();
                $correct_array  = array();
                $answer_default = '';
                $query_default  = '';
                $textarea_size = 0;

                if ($exercise_type == 'sc_exercise' || $exercise_type == 'sco_exercise')
                {
                        foreach ($answers as $key => $node) {
                                $group = $node->get_attribute("group");
                                if ($group == '') $group = 0;

                                $answer_array[$group][] = html_ready(utf8_decode($node->get_content()));
                                if ($node->get_attribute("Correct") == '1')
                                    $correct_array[$group] = count($answer_array[$group]) - 1;
                        }
                }
                elseif ($exercise_type == 'yn_exercise')
                {
                        foreach ($answers as $key=>$node) {
                                $answer_array[]         = html_ready(utf8_decode($node->get_content()));
                                if ($node->get_attribute("Correct")==1)
                                        $correct_array[0]       = $key;
                        }

                        $label_yes = $answer_array[0];
                        $label_no  = $answer_array[1];
                }
                elseif ($exercise_type == 'mc_exercise'||
                                $exercise_type == 'mco_exercise'||
                                $exercise_type == 'lt_exercise')
                {
                        foreach ($answers as $node) {
                                $answer_array[]         = html_ready(utf8_decode($node->get_content()));
                                $correct_array[]        = $node->get_attribute("Correct");
                        }
                        if ($exercise_type == 'lt_exercise') {
                                $answer_distance        = $o_exercise->getTagsFromXML($xml, "AnswerDistance");
                                $answer_distance        = $answer_distance[0];
                        }

                        $label_yes = _('Ja');
                        $label_no  = _('Nein');

                        if ($exercise_type == 'mco_exercise') {
                                $labels = $dom_xml->get_elements_by_tagname("AnswerLabel");
                                foreach ($labels as $node) {
                                    if ($node->get_attribute("Correct") == '1')
                                        $label_yes = utf8_decode($node->get_content());
                                    else
                                        $label_no  = utf8_decode($node->get_content());
                                }
                        }
                }
                elseif ($exercise_type == 'tb_exercise'||
                                $exercise_type == 'pl_exercise'||
                                $exercise_type == 'oc_exercise'||
                                $exercise_type == 'oct_exercise')
                {
                        $node = $answers[0];
                        $answer_array[0]        = html_ready(utf8_decode($node->get_content()));

                        if ($exercise_type == 'tb_exercise') {
                                $answer_default         = $o_exercise->getTagsFromXML($xml, "AnswerDefault");
                                $answer_default         = html_ready($answer_default[0]);
                                $answer_distance        = $o_exercise->getTagsFromXML($xml, "AnswerDistance");
                                $answer_distance        = $answer_distance[0];
                        }
                        elseif ($exercise_type == 'pl_exercise') {
                                $answer_default         = $o_exercise->getTagsFromXML($xml, "AnswerDefault");
                                $answer_default         = html_ready($answer_default[0]);
                                $query_default          = $o_exercise->getTagsFromXML($xml, "QueryDefault");
                                $query_default          = html_ready($query_default[0]);

                        }
                }
                
                // rnd_exercise //
                else if ($exercise_type == "rnd_exercise")
                {
                        
                        /* fetch stored subexercises */
                        $sub = $dom_xml->get_elements_by_tagname("SubExercise");
                        foreach ($sub as $ex)
                        {
                                $subexercises[] = $ex->get_attribute("ID");
                        }
                        
                        if (count($subexercises))
                        {
                                $query = "SELECT ID, Name, URI
                                        FROM `vips_aufgabe`
                                        WHERE ID = '".array_shift($subexercises)."'";
                                foreach ($subexercises as $sub)
                                {
                                        $query .= " OR ID = '".$sub."' ";
                                }
                                $result = $vips_db->executeQuery($query);
                                
                                while ($result->next())
                                {
                                        $subexercises_array[] = array("id" => $result->get("ID"), "name" => $result->get("Name"), "type" => $result->get("URI"));
                                }
                        }
                        
                        $subexercises_number = count($subexercises);
                        
                        
                        
                        /* fetch all exercise types */
                        $query = "SELECT Typenname, URI FROM vips_aufgaben_typen";
                        $result = $vips_db->executeQuery($query);
                        while ($result->next())
                        {
                                // don't let the lecturer choose a rnd_exercise as a subexercise of a rnd_exercise...
                                if ($result->get("URI") == "rnd_exercise") continue;
                                $exercise_types[] = array("Name" => $result->get("Typenname"), "URI" => $result->get("URI"));
                        }
                        
                        
                        
                        /* fetch all already existing exercises of this lecturer */
                        $query = "SELECT ID, Name, URI
                                  FROM vips_aufgabe
                                  WHERE Userid = '".$g_outlet->userID."'";
                        $result = $vips_db->executeQuery($query);
                        while ($result->next())
                        {
                                // don't let the lecturer choose a rnd_exercise as a subexercise of a rnd_exercise...
                                if ($result->get("URI") == "rnd_exercise") continue;
                                $existing_exercises[] = array("Name"=> $result->get("Name"), "ID" => $result->get("ID"));
                        }
                }
                
                // rh_exercise //
                else if ($exercise_type == "rh_exercise")
                {
                        foreach ($answers as $node)
                        {
                                $answer_array[] = utf8_decode($node->get_content());
                                $to_default_array[] = $node->get_attribute("Default");
                        }
                        
                        foreach ($dom_xml->get_elements_by_tagname("Default") as $default)
                        {
                                $default_array[] = utf8_decode($default->get_content());
                        }
                }
                
                // zu_exercise //
                elseif ($exercise_type == 'zu_exercise') {
                        /*$exercise_picurl = $o_exercise->getTagsFromXML($xml, "PictureURL");
                        $exercise_picurl = urldecode($exercise_picurl[0]);
                        foreach ($answers as $node) {
                                $solution = array();
                                $solution[0] = $node->get_attribute("x1"); 
                                $solution[1] = $node->get_attribute("y1"); 
                                $solution[2] = $node->get_attribute("x2"); 
                                $solution[3] = $node->get_attribute("y2"); 
                                $answer_array[]         = urldecode(utf8_decode($node->get_content()));
                                $correct_array[]        = $solution;
                        }*/
                }

                // if I don't come from add_answer show "real", existing answers
                if (getExternalParam('add_answer') == '' && getExternalParam('add_answer_x') == '' &&
                    getExternalParam('add_group') == '' && getExternalParam('add_group_x') == '') {
                        if (is_array($answer_array[0])) {
                                $number_of_answers = implode(',', array_map('count', $answer_array));
                        } else {
                                $number_of_answers = count($answer_array);
                        }
                }
    }
    
    /**************************************************************************/
    $t_header = $vipsTemplateFactory->open('exercises/edit_exercise_header');
    $t_main   = $vipsTemplateFactory->open('exercises/edit_'.$exercise_type);
    $t_footer = $vipsTemplateFactory->open('exercises/edit_exercise_footer');
    /**************************************************************************/
    //if $exercise_id is empty in the template, the system knows that it has 
    //to create a new exercise:
    if ($p_copy) {
        $exercise_id = '';  // to create a new instance of an existing exercise
    }

    $t_header->set_attributes(array(
        'picturepath'           => $g_outlet->picturepath,
        'dialog'                => $dialog,
        'action'                => $p_action,
        'assignment_id'         => $assignment_id,
        'exercise_title'        => $exercise_typenname,
        'exercise_id'           => $exercise_id,
        'exercise_position'     => $exercise_position,
        'exercise_name'         => $exercise_name,
        'exercise_type'         => $exercise_type,
        'exercise_question'     => $exercise_question,
        'number_of_exercises'   => $number_of_exercises,
        'number_of_answers'     => $number_of_answers,
        "in_rnd_exercise"       => $in_rnd_exercise,
        "rnd_exercise_id"       => $rnd_exercise_id,
        "rnd_exercise_name"     => $rnd_exercise_name,
        'rnd_exercise_position' => $rnd_exercise_position));
    /**************************************************************************/
    $t_main->set_attributes(array(
        'picturepath'           => $g_outlet->picturepath,
        'action'                => $p_action,
        'exercise_id'           => $exercise_id,
        'exercise_name'         => $exercise_name,
        'answer_array'          => $answer_array,
        'correct_array'         => $correct_array,
        'default_array'         => $default_array,        // for rh
        'to_default_array'      => $to_default_array,     // for rh
        'answer_distance'       => $answer_distance,      // for tb + lt
        'answer_default'        => $answer_default,       // for tb + pl
        'query_default'         => $query_default,        // for pl
        'textarea_size'         => $textarea_size,        // for pl
    //$t_main->set_attribute('exercise_picurl',     $exercise_picurl);  //for zu
        'mc_auswertung'         => $mc_auswertung,        // for mc
        'label_yes'             => $label_yes,            // for yn + mco
        'label_no'              => $label_no,             // for yn + mco
        'number_of_answers'     => $number_of_answers,
        'subexercises_number'   => $subexercises_number,  // for rnd
        'exercise_types'        => $exercise_types,       // for rnd
        'existing_exercises'    => $existing_exercises,   // for rnd
        'subexercises_array'    => $subexercises_array,   // for rnd
    //$t_main->set_attribute("rnd_exercise_data", $rnd_exercise_data);
        'rnd_exercise_position' => $exercise_position,
        'assignment_id'         => $assignment_id));
    /**************************************************************************/
    $t_footer->set_attributes(array(
        'exercise_hint'         => $exercise_hint,
        'exercise_comment'      => $exercise_comment,
        'picturepath'           => $g_outlet->picturepath,
        'action'                => $p_action,
        'assignment_id'         => $assignment_id,
        'in_rnd_exercise'       => $in_rnd_exercise,
        'rnd_exercise_id'       => $rnd_exercise_id,
        'rnd_exercise_name'     => $rnd_exercise_name,
        'rnd_exercise_position' => $rnd_exercise_position,
        'exercise_id'           => $exercise_id,
        'exercise_type'         => $exercise_type,
        'exercise_position'     => $exercise_position,
        'all_exercise_types'    => $all_exercise_types,
        'number_of_exercises'   => $number_of_exercises));
    /**************************************************************************/
    
    echo $t_header->render();
    echo $t_main->render();
    echo $t_footer->render();
}


/**
 * SHEETS/EXAMS
 * 
 * Inserts/Updates an exercise into the database
 */

function store_exercise($p_action) {
    global $vips_db,
        $g_outlet,
        $g_inAssignment_table,
        $g_assignment_table;
    
    $assignment_id       = getExternalParam('assignment_id');
    $exercise_id         = $_POST['exercise_id'];                        //BUG???: $_GET['exercise_id'] kann belegt sein, auch bei Neuanlegen!!!
    $exercise_type       = getExternalParam('exercise_type');
    $exercise_name       = getExternalParam('exercise_name');
    $exercise_position   = getExternalParam("exercise_position");
    
    /*$exercise_question = getExternalParam('exercise_question');
    $exercise_hint       = getExternalParam('exercise_hint');
    $exercise_comment    = getExternalParam('exercise_comment');
    */
    $number_of_answers   = getExternalParam('number_of_answers');  //wrong if mc/mco-exercise number of answers has been changed!!!
    $number_of_exercises = getExternalParam('number_of_exercises');
    
        $dialog = array();
        $num_answers = explode(',', $number_of_answers);

        for ($group = 0; $group < count($num_answers); ++$group) {
            if (getExternalParam('add_answer_'.$group) || getExternalParam('add_answer_'.$group.'_x') != '') {
                // NOTE: must be a string because of incredible brokenness of getExternalParam()
                $_POST['add_answer'] = "$group";
                break;
            }
        }

        if (getExternalParam('add_answer') != '' || getExternalParam('add_answer_x') != '') {           //add a new answer field
            if (count($num_answers) == 1) {
                ++$number_of_answers;
            } else {
                ++$num_answers[$group];
                $number_of_answers = implode(',', $num_answers);
            }
        }
        elseif (getExternalParam('add_group') != '' || getExternalParam('add_group_x') != '') {
            $num_answers[] = 3;
            $number_of_answers = implode(',', $num_answers);
        }
        /* For random exercises: check if "create" button has been pressed and a */
        /* valid exercise type has been chosen for creation. */
        elseif ($exercise_type == "rnd_exercise" && (getExternalParam("create") || getExternalParam("create_x") != '') && getExternalParam("exercise_types") != "NULL")
        {
            $_POST["new_exercise"] = 1;  // flag to indicate that a new exercise is to be created
            $_POST["new_exercise_type"] = getExternalParam("exercise_types");  // from dropdown field
            
            // some additional parameters to steer that the new exercise is linked
            // to random exercise and not to assignment sheet
            $_POST["in_rnd_exercise"] = 1;  // flag: values 0,1
            $_POST["rnd_exercise_id"] = $exercise_id;  // TODO Ob das wohl klappt, wenn die rnd-Aufgabe noch keine ID hat, also ganz am Anfang vom Anlegen? Wohl nicht.... Mal was ausdenken.
            $_POST["rnd_exercise_name"] = $exercise_name;
            $_POST["rnd_exercise_position"] = $exercise_position;
        }
        else {
            $dialog['name']         = 'exercise_stored';
            $dialog['mode']         = 'msg';
            $dialog['message']      = 'Die Aufgabe wurde eingetragen';
        }
        
        /*** store exercise */

        /*** File upload for zu_exercise */
        if ($exercise_type      == 'zu_exercise') {
                /* TODO copy code from studip_impuls
                 
                if (getExternalParam('change_picurl'))
                        $_POST['exercise_picurl'] = '';
                else
                        for ($i = 0; $i<$number_of_answers;$i++)
                                if (getExternalParam("delete_answer_$i"."-0"))
                                        $_POST["answer_$i-0"] = '';
                //$path = "/$g_outlet->userID/";
                //if (mkdir($path, 0777)) echo "success!";
                //else echo "no directory";
                if($_FILES['exercise_picurl']) {
                                // upload of main picture 

                        if(is_uploaded_file($_FILES['exercise_picurl']['tmp_name'])) {
                                $imageInfo = getimagesize($_FILES['exercise_picurl']['tmp_name']);
                                $width = $imageInfo[0];
                                $height = $imageInfo[1];
                                $image_type = $imageInfo[2];
                                //print_r($imageInfo );

                                move_uploaded_file($_FILES['exercise_picurl']['tmp_name'], "vips/upload/".$_FILES['exercise_picurl']['name']);
                        }
                        else  echo "Fehler beim Hochladen des Hintergrundbildes!";
                }

                for ($i=0; $i<$number_of_answers;$i++) {
                        if($_FILES["answer_$i-0"]) {
                                if(is_uploaded_file($_FILES["answer_$i-0"]['tmp_name']))
                                {
                                        $imageInfo = getimagesize($_FILES["answer_$i-0"]['tmp_name']);
                                        $width = $imageInfo[0];
                                        $height = $imageInfo[1];
                                        $image_type = $imageInfo[2];
                                        //print_r($imageInfo );

                                        move_uploaded_file($_FILES["answer_$i-0"]['tmp_name'], "vips/upload/".$_FILES["answer_$i-0"]['name']);
                                }
                                else { $i_temp = $i+1; echo "Fehler beim Hochladen von Bild".$i_temp."!"; }
                        }
                }*/
        } // end if exercise_type=zu_exercise

                
        /***********************************************************/
        require_once("exercises/$exercise_type.php");
        $o_exercise = new $exercise_type();
        /***********************************************************/
        
        
        
        ////////////////////////
        // fetch exercise XML //
        ////////////////////////
        
        $xml = $o_exercise->getExerciseConstruct();  //create xml with question, answers...
        $exercise_points = $o_exercise->g_maxPoints;
        
        // TODO Sollte ausgelagert werden in die getExerciseConstruct der rh_exercise!
        if ($exercise_type == "rh_exercise")
        {
                $special_action = false;
                
                if (getExternalParam("add_pairs") || getExternalParam("add_pairs_x") != '')
                {
                        $special_action = true;
                        $number_to_add = getExternalParam("number_to_add");
                        $xml = $o_exercise->add_pairs($xml, $number_to_add);
                }
                else if (getExternalParam("mix") || getExternalParam("mix_x") != '')
                {
                        $special_action = true;
                        $xml = $o_exercise->mix_answers($xml);
                }
                else
                {
                        // check if answers shall be moved or pairs shall be deleted
                        for ($i = 0; $i < $number_of_answers; $i++)
                        {
                                if (getExternalParam("move_up_answer_".$i) || getExternalParam("move_up_answer_".$i."_x") != '')
                                {
                                        $special_action = true;
                                        $xml = $o_exercise->move_answer($xml, $i, "up");
                                }
                                else if (getExternalParam("move_down_answer_".$i) || getExternalParam("move_down_answer_".$i."_x") != '')
                                {
                                        $special_action = true;
                                        $xml = $o_exercise->move_answer($xml, $i, "down");
                                }
                                else if (getExternalParam("delete_pair_".$i) || getExternalParam("delete_pair_".$i."_x") != '')
                                {
                                        $special_action = true;
                                        $xml = $o_exercise->delete_pairs($xml, $i);
                                }
                        }
                }
                
                // if no special action was executed, ie. if "save" button has been pressed
                if ($special_action === false)
                {
                        $xml = $o_exercise->delete_pairs($xml);  // delete all empty pairs
                }
        }
        
        
        
        /*** store already existing exercise */
        if ($exercise_id) {
                // Update name and XML for this exercise.
                $query = "UPDATE
                        vips_aufgabe
                    SET
                        Name = '$exercise_name',
                        Aufgabe = '$xml'
                    WHERE
                        ID = '$exercise_id'
                    ";
                $vips_db->executeUpdate($query);
                                
                //bei mc und mco Aufgaben bestimmt die Anzahl der nicht-leeren Antwortalternativen die Gesamtpunktzahl
                // - Auch bei rh-Aufgaben!
                // TODO perhaps the exercise class should have counted the max points correcty in the first place...
                if ($exercise_type == "mc_exercise" || $exercise_type == "mco_exercise" || $exercise_type == "rh_exercise")
                {
                        $i = 0;
                        while (getExternalParam("answer_$i")!='') {
                                $i++;
                        }
                        // Update maximum points.
                        // TODO marschro Was passiert hier bei Aufgaben, die nur in einer rnd_exercise stecken und nicht im Aufgabenblatt?
                        $vips_db->executeUpdate("UPDATE $g_inAssignment_table
                                            SET Punkte = '$i'
                                            WHERE vips_aufgabe   = '$exercise_id'
                                            AND $g_assignment_table = '$assignment_id'");
                }
                elseif ($exercise_type == "sc_exercise" || $exercise_type == "sco_exercise") {
                        // Update maximum points.
                        $vips_db->executeUpdate("UPDATE $g_inAssignment_table
                                            SET Punkte = '$exercise_points'
                                            WHERE vips_aufgabe   = '$exercise_id'
                                            AND $g_assignment_table = '$assignment_id'");
                }
        }
        /*** store new exercise */
        else {
                // find out max Position
                $result = $vips_db->executeQuery(
                        "SELECT MAX(Position) AS pos FROM $g_inAssignment_table".
                        " WHERE $g_assignment_table = '$assignment_id'");
                $result->next();
                $exercise_position = $result->getInt('pos') + 1;

                // Store exercise in database.
                $query = "INSERT INTO vips_aufgabe
                                        SET
                                                Name = '$exercise_name',
                                                Aufgabe = '$xml',
                                                URI = '$exercise_type',
                                                Userid = '$g_outlet->userID'
                                ";
                $vips_db->executeUpdate($query);
                $vips_idgen = $vips_db->getIdGenerator();
                $exercise_id = $vips_idgen->getId();
                
                
                //bei mc und mco Aufgaben bestimmt die Anzahl der nicht-leeren Antwortalternativen die Gesamtpunktzahl
                // - Auch bei rh-Aufgaben!
                // TODO perhaps the exercise class should have counted the max points correcty in the first place...
                if ($exercise_type == "mc_exercise" || $exercise_type == "mco_exercise" || $exercise_type == "rh_exercise")
                {
                        $i = 0;
                        while (getExternalParam("answer_$i")!='') {
                                $i++;
                        }
                        $exercise_points = $i;  
                }
                
                
                
                // Add new exercise as subexercise to random exercise.
                if (getExternalParam("in_rnd_exercise"))
                {
                        require_once("exercises/rnd_exercise.php");
                        $rnd0_exercise = new rnd_exercise();
                        
                        $rnd_exercise_id = getExternalParam("rnd_exercise_id");
                        
                        // Fetch rnd_exercise-XML from database.
                        $query = "SELECT `Aufgabe`
                                FROM `vips_aufgabe`
                                WHERE ID = '".$rnd_exercise_id."'
                                ";
                        $result = $vips_db->executeQuery($query);
                        $result->next();
                        $rnd_xml = $result->get("Aufgabe");
                        
                        $rnd_xml = $rnd0_exercise->appendNewSubexercise($rnd_xml, $exercise_id);
                        
                        // Store extended XML for rnd_exercise.
                        $query = "UPDATE `vips_aufgabe`
                                SET Aufgabe = '$rnd_xml'
                                WHERE ID = '".$rnd_exercise_id."'
                                        AND URI = 'rnd_exercise'
                                        AND Userid = '".$g_outlet->userID."'
                                ";
                        $vips_db->executeUpdate($query);
                }
                
                // Link new exercise to the assignment.
                else
                {
                        $query2 = "INSERT INTO $g_inAssignment_table
                                        SET
                                                vips_aufgabe = '$exercise_id',
                                                $g_assignment_table = '$assignment_id',
                                                Punkte = '$exercise_points',
                                                Bewertet = 't',
                                                Position = '$exercise_position'
                                ";
                        $vips_db->executeUpdate($query2);
                }
        }
    
    $_POST['assignment_id']       = $assignment_id;
    $_POST['exercise_id']         = $exercise_id;  // necessary if target is for edit_exercise
    $_POST['exercise_type']       = $exercise_type;
    $_POST['exercise_position']   = $exercise_position;
    $_POST['number_of_exercises'] = $number_of_exercises;
    $_POST['number_of_answers']   = $number_of_answers;
    $_POST['dialog']              = $dialog;
    $_POST['in_rnd_exercise']     = getExternalParam('in_rnd_exercise');
    
    edit_exercise($p_action);  // show the same exercise
}

/**
 * Copy the selected exercises into this assignment.
 */
function copy_exercise ($action, $use_ajax)
{
    global $vips_db, $g_outlet, $g_inAssignment_table, $g_assignment_table;

    $assignment_id = getExternalParam('assignment_id');
    $exercise_ids  = getExternalParam('exercise_ids');

    // find out max Position
    $result = $vips_db->executeQuery(
            "SELECT MAX(Position) AS pos FROM $g_inAssignment_table".
            " WHERE $g_assignment_table = '$assignment_id'");
    $result->next();
    $exercise_position = $result->getInt('pos');

    $vips_idgen = $vips_db->getIdGenerator();

    foreach ($exercise_ids as $exercise_id)
    {
        ++$exercise_position;
        $vips_db->executeUpdate("INSERT INTO vips_aufgabe (Name, Aufgabe, URI, Userid)
                                     SELECT Name, Aufgabe, URI, '$g_outlet->userID'
                                     FROM vips_aufgabe WHERE ID = '$exercise_id'");
        $new_id = $vips_idgen->getId();

        $result = $vips_db->executeQuery("SELECT * FROM $g_inAssignment_table WHERE vips_aufgabe = '$exercise_id'");
        $result->next();
        $punkte = $result->getFloat('Punkte');

        $vips_db->executeUpdate("INSERT INTO $g_inAssignment_table (vips_aufgabe, $g_assignment_table, Punkte, Bewertet, Position)
                                 VALUES ($new_id, '$assignment_id', $punkte, 't', $exercise_position)");
    }

    if ($use_ajax) {
        list_exercises_ajax($action, $assignment_id);
    } else {
        edit_assignment($action);
    }
}

/**
 * Exports all exercises in this aassignment in a plain text format.
 */
function export_assignment ($action)
{
    global $vips_db, $g_outlet, $g_inAssignment_table, $g_assignment_table, $g_title_s;

    $assignment_id = getExternalParam('assignment_id');

    if ($g_outlet->status == 'student')
    {
        $message['mode'] = 'error';
        $message['message'] = _('Zugriff nur fr Dozenten erlaubt.');
        $_POST['message'] = $message;

        return list_assignments_stud($action);
    }

    #######################
    # get assignment data #
    #######################

    $query = "SELECT * FROM $g_assignment_table WHERE ID = '$assignment_id'";
    $result = $vips_db->executeQuery($query);
    $result->next();

    $course_name      = html_entity_decode(getHeaderLine($g_outlet->courseID), ENT_QUOTES);
    $assignment_name  = $result->getString('Name');
    $assignment_start = $result->getString('Beginn');

    #####################
    # get all exercises #
    #####################

    $query = "SELECT vips_aufgabe.*
              FROM vips_aufgabe, $g_inAssignment_table
              WHERE $g_assignment_table = '$assignment_id'
                  AND vips_aufgabe = vips_aufgabe.ID
              ORDER BY Position";

    $result = $vips_db->executeQuery($query);

    // this hack is necessary to disable the standard Stud.IP layout
    ob_end_clean();
    header('Content-Type: text/plain; charset=ISO-8859-1');
    header('Content-Disposition: attachment; filename="'.$assignment_name.'.txt"');
    header('Pragma: public');

    echo "# $course_name\n";
    echo "# $g_title_s: $assignment_name\n";
    echo "# Beginn: $assignment_start\n\n";

    while ($result->next())
    {
        $exercise_type      = $result->getString('URI');
        $exercise_xml       = $result->getString('Aufgabe');

        require_once("exercises/$exercise_type.php");

        $exercise = new $exercise_type();
        echo $exercise->exportText($exercise_xml);
        echo "\n";
    }

    ob_start('discard_buffer');
}

/**
 * SHEETS/EXAMS
 * 
 * Stores the specification (Grunddaten) of an assignment
 * OR add new exercise, edit Punkte/Bewertung (basically everything that can be done on
 * page edit_exercise())
 */

function store_assignment($p_action) {
    global $vips_db,
        $g_outlet,
        $g_inAssignment_table,
        $g_assignment_table;
    
    $assignment_id          = getExternalParam('assignment_id');
    $assignment_name        = getExternalParam('assignment_name');
    $assignment_description = getExternalParam('assignment_description');
    $start_day              = getExternalParam('start_day');
    $start_month            = getExternalParam('start_month');
    $start_year             = getExternalParam('start_year');
    $start_h                = getExternalParam('start_h');
    $start_m                = getExternalParam('start_m');
    $end_day                = getExternalParam('end_day');
    $end_month              = getExternalParam('end_month');
    $end_year               = getExternalParam('end_year');
    $end_h                  = getExternalParam('end_h');
    $end_m                  = getExternalParam('end_m');
    
    $exam_length            = getExternalParam('exam_length');
    
    $number_of_exercises    = getExternalParam('number_of_exercises');
    $selftest               = getExternalParam('selftest');
    $printable              = getExternalParam('printable') ? 1 : 0;
    $ip_range               = getExternalParam('ip_range');
    $mc_auswertung          = getExternalParam('mc_auswertung');

    if (getExternalParam('store') || getExternalParam('store_x') != '')         
    {
        // find out max Position
        $result = $vips_db->executeQuery("SELECT MAX(Position) AS pos FROM $g_assignment_table WHERE Kurs = '$g_outlet->courseID'");
        $result->next();
        $position = $result->getInt('pos') + 1;     // TODO: use getRecordCount() here, position of assignment not yet implemented

        $start_unix_ts = mktime($start_h, $start_m, 0, $start_month, $start_day, $start_year); //$start = "$start_year-$start_month-$start_day $start_h:$start_m:00";
        $start         = date("Y-m-d H:i:s", $start_unix_ts);
        $end_unix_ts   = mktime($end_h, $end_m, 0, $end_month, $end_day, $end_year); //$end = "$end_year-$end_month-$end_day $end_h:$end_m:00";
        $end           = date("Y-m-d H:i:s", $end_unix_ts);
        $length        = ($end_unix_ts - $start_unix_ts) / 60;
        
        if ($length <= 0)  // start is *later* than end!
        {
            $end_unix_ts      = $start_unix_ts;
            $end              = $start;
            $message['mode']  = 'info';
            $message['message'] = _('Bitte berprfen Sie den Start- und den Endzeitpunkt!');
            $_POST['message'] = $message;
        }
        
        if ($p_action == 'exams')
        {
            $length = $exam_length;  // as specified above
        }

        /*** store basic data (Grunddaten) of assignment */
        if ($assignment_id) {
            // check whether the exam's start time has been moved
            $query = "SELECT Beginn FROM $g_assignment_table WHERE ID = '$assignment_id'";
            $result = $vips_db->executeQuery($query);
            $result->next();
            
            // delete start times for users which already have begun to solve the exam
            if ($result->get("Beginn") != $start)
            {
                $query = "DELETE FROM vips_aufgaben_zeit
                    WHERE Kurs = '$g_outlet->courseID'
                    AND $g_assignment_table = '$assignment_id'";
                $vips_db->executeUpdate($query);
            }
            
            $query = "UPDATE $g_assignment_table
                SET Name = '$assignment_name',
                    Beginn = '$start',
                    Ende = '$end',
                    Dauer = '$length',
                    Thema = '$assignment_description',
                    Selbsttest = '$selftest',
                    Sichtbarkeit = '$printable',
                    IPBereich = '$ip_range',
                    mc_auswertung = '$mc_auswertung'
                WHERE
                    ID = '$assignment_id'";
            $vips_db->executeUpdate($query);
        }
        else
        {
            $query = "INSERT INTO $g_assignment_table
                SET Name = '$assignment_name',
                    Beginn = '$start',
                    Ende = '$end',
                    Kurs = '$g_outlet->courseID',
                    Dauer = '$length',
                    Thema = '$assignment_description',
                    Selbsttest = '$selftest',
                    Sichtbarkeit = '$printable',
                    IPBereich = '$ip_range',
                    Position = '$position',
                    mc_auswertung = '$mc_auswertung'";
            $vips_db->executeUpdate($query);
            $vips_idgen = $vips_db->getIdGenerator();
            $assignment_id = $vips_idgen->getId();
        }
        
        $points = $_POST['exercise_points_array'];      // [1,3,5,4,3,3,3,3,...]
        $eval   = $_POST['exercise_eval_array'];        // [f,t,t,f,t,f,t,f,...]
        for ($i = 0; $i < $number_of_exercises; $i++) {
            if (!$eval[$i]) $eval[$i] = 't';
            $points[$i] = (float) strtr($points[$i], ',', '.');
            $points[$i] = round_to_half_point($points[$i]);
            
            $query = "UPDATE $g_inAssignment_table
                SET Punkte = '$points[$i]',
                    Bewertet = '$eval[$i]'
                WHERE $g_assignment_table='$assignment_id'
                    AND Position = ".($i+1);
            $vips_db->executeUpdate($query);
        }
        
        $_POST['assignment_id'] = $assignment_id;
        
        edit_assignment($p_action);  // show the same template from edit PLUS vv
    }
    else if (getExternalParam('create_exercise') || getExternalParam('create_exercise_x') != '')        
    {
        /*** create a new exercise */
        $_POST['new_exercise'] = 1;

        edit_exercise($p_action);
    }
    else if (getExternalParam('copy_exercise') || getExternalParam('copy_exercise_x') != '')               
    {
        /*** copy existing exercise */
        copy_exercise($p_action, false);
    }
    else if (getExternalParam('simulate_assignment') || getExternalParam('simulate_assignment_x') != '')
    {
        /*** display assignment as if for students */
        list_assignments_stud($p_action);
    }
    else if (getExternalParam('export_assignment') || getExternalParam('export_assignment_x') != '')
    {
        /*** export assignment to text file */
        export_assignment($p_action);
    }
    else if (getExternalParam('clear_search') || getExternalParam('clear_search_x') != '')
    {
        /*** clear current search */
        $_POST['search'] = '';

        edit_assignment($p_action);
    }
    else /* no parameter name (JavaScript form submit) */
    {
        /*** search existing exercises */
        edit_assignment($p_action);
    }
}



/**
 * SHEETS/EXAMS
 * 
 * Displays form to edit an existing assignment
 *
 */

function edit_assignment($p_action) {
    global $vips_db,
        $g_outlet,
        $g_title_s,
        $g_title_p,
        $g_assignment_table,
        $g_inAssignment_table,
        $vipsPlugin,
        $vipsTemplateFactory,
        $auth,
        $SOFTWARE_VERSION;
    
    $assignment_id  = getExternalParam('assignment_id');
    $message        = getExternalParam('message');
    $sem_start_time = getExternalParam('start_time');
    $test_id        = getExternalParam('test_id');
    $search         = getExternalParam('search');
    
    #######################
    # get assignment data #
    #######################
    
    $query = "SELECT * FROM $g_assignment_table WHERE ID = '$assignment_id'";
    $result = $vips_db->executeQuery($query);
    $result->next();
    $entry = $result->getRow();
    
    list($start_date, $start_time) = split(" ", $entry['Beginn']);
    list($start_year, $start_month, $start_day) = split("-", $start_date);
    list($start_h, $start_m, $start_s) = split(":", $start_time);
    
    list($end_date, $end_time) = split(" ", $entry['Ende']);
    list($end_year, $end_month, $end_day) = split("-", $end_date);
    list($end_h, $end_m, $end_s) = split(":", $end_time);
    
    $assignment_id          = $entry['ID'];
    $assignment_name        = $entry['Name'];
    $assignment_description = $entry['Thema'];
    $length                 = $entry['Dauer'];
    $selftest               = $entry['Selbsttest'];
    $printable              = $entry['Sichtbarkeit'];
    $ip_range               = $entry['IPBereich'];
    $mc_auswertung          = $entry['mc_auswertung'];
    
    $t_main = $vipsTemplateFactory->open('edit_assignment');
    $t_main->set_attributes(array(
        'picturepath'            => $g_outlet->picturepath,
        'vips_imagepath'         => $vipsPlugin->getPluginPath().'/images/',
        'use_ajax'               => vips_use_ajax(),
        'g_title_s'              => $g_title_s,
        'g_title_p'              => $g_title_p,
        'action'                 => $p_action,
        'message'                => $message,
        'assignment_id'          => $assignment_id,
        'assignment_name'        => $assignment_name,
        'assignment_description' => $assignment_description,
        'end_year'               => $end_year,
        'end_month'              => $end_month,
        'end_day'                => $end_day,
        'end_h'                  => $end_h,
        'end_m'                  => $end_m,
        'start_year'             => $start_year,
        'start_month'            => $start_month,
        'start_day'              => $start_day,
        'start_h'                => $start_h,
        'start_m'                => $start_m,
        'selftest'               => $selftest,
        'printable'              => $printable,
        'ip_range'               => $ip_range,
        'mc_auswertung'          => $mc_auswertung,
        'exam_length'            => $length,
        'start_time'             => $sem_start_time,
        'test_id'                => $test_id,
        'search'                 => $search
    ));
    
    // After an assignment has expired, the lecturer is only allowed to adjust
    // points and to mark an exercise as not to be evaluated. He must neither
    // add, remove or edit exercises nor change the exercise order!
    /* disabled for now, see http://plugins.studip.de/trac/ticket/32
    $now = date("Y-m-d H:i:s");
    if ($now > $entry["Ende"]) {
        $running_status = "expired";
    } else if ($now < $entry["Beginn"]) {
        $running_status = "pending";
    } else {
        $running_status = "running";
    }
    $t_main->set_attribute("running_status", $running_status);
    */
    
    
    #############################################
    # get existing exercises in this assignment #
    #############################################
    
    $query = "SELECT vips_aufgabe.*,
            vips_aufgaben_typen.Typenname,
            $g_inAssignment_table.Position,
            $g_inAssignment_table.Punkte,
            $g_inAssignment_table.Bewertet
        FROM vips_aufgabe,
            vips_aufgaben_typen,
            $g_inAssignment_table
        WHERE $g_assignment_table = '$assignment_id'
            AND vips_aufgabe = vips_aufgabe.ID
            AND vips_aufgaben_typen.URI = vips_aufgabe.URI
        ORDER BY $g_inAssignment_table.Position
        ";
    $result = $vips_db->executeQuery($query);

    $exercises = array();
    while ($result->next()) {
        $entry = $result->getRow();
        $entry['Punkte'] = $result->getFloat('Punkte');
        $exercises[] = $entry;
    }
    $t_main->set_attribute('exercises', $exercises);
    
    $number_of_exercises = $result->getRecordCount();
    $t_main->set_attribute('number_of_exercises', $number_of_exercises);
    
    
    #######################################
    # get exercise types for new exercise #
    #######################################
    
    $query = "SELECT * FROM vips_aufgaben_typen";
    $result = $vips_db->executeQuery($query);
    $exercise_types = array();
    while ($result->next())
    {
        $entry = $result->getRow();
        $exercise_types[] = $entry;
    }
    $t_main->set_attribute('exercise_types', $exercise_types);
    
    // get semester, course, assignment and exercise data for copying.
    // available are all assignments that all lecturers in this course
    // have access to (including other courses)

    $t_main->set_attribute('semesters', get_accessible_semesters());

    if ($start_time != '') {
        $t_main->set_attribute('courses', get_accessible_courses($sem_start_time));
    }

    if ($test_id != '') {
        $t_main->set_attribute('all_exercises', get_exercises($test_id));
    }

    if ($search != '') {
        $t_main->set_attribute('search_result', get_matching_exercises($search));
    }

    setlocale(LC_NUMERIC, NULL);
    echo $t_main->render();
    setlocale(LC_NUMERIC, 'C');
}

/**
 * Show preview of an existing exercise (using print view for now).
 */
function preview_exercise ($action)
{
    vips_output_begin();

    $_POST['print'] = 1;
    show_exercise($action);

    vips_output_end();
}

/**
 * SHEETS/EXAMS
 * 
 * Copies assignment completely, i.e. inserts new assignment plus copied exercises from old
 * assignment
 * 
 */

function copy_assignment($p_action) {
    global $vips_db,
        $g_outlet,
        $g_inAssignment_table,
        $g_assignment_table;
    
    $assignment_id  = getExternalParam('assignment_id');
    $message = array();
    
    #################################
    # copy and paste new assignment #
    #################################
    
    $vips_db->executeUpdate("INSERT INTO $g_assignment_table (Name,Beginn,Ende,Dauer,Selbsttest,IPBereich,Position,Thema,istGesperrt,korrekturenSichtbar)
        SELECT Name,Beginn,Ende,Dauer,Selbsttest,IPBereich,Position,Thema,istGesperrt,korrekturenSichtbar
        FROM $g_assignment_table
        WHERE ID = '$assignment_id'");
    $vips_idgen = $vips_db->getIdGenerator();
    $new_assignment_id = $vips_idgen->getId();
    
    // find out max Position
    $sql = "SELECT MAX(Position) AS pos FROM $g_assignment_table WHERE `Kurs` = '$g_outlet->courseID'";
    $result = $vips_db->executeQuery($sql);
    $result->next();
    $new_assignment_position = $result->getInt('pos') + 1;
    
    // Change name, course and position of copy
    $sql = "UPDATE $g_assignment_table
        SET Name     = CONCAT('"._('Kopie von ')."',Name),
            Kurs     = '$g_outlet->courseID',
            Position = $new_assignment_position
        WHERE ID = '$new_assignment_id'";
    $vips_db->executeUpdate($sql);
    
    ####################################################################
    # copy and paste all exercises associated with original assignment #
    # and copy and paste info from inAssignment table                  #
    ####################################################################
    
    $sql = "SELECT vips_aufgabe.*
        FROM vips_aufgabe,
            $g_inAssignment_table
        WHERE vips_aufgabe.ID = $g_inAssignment_table.vips_aufgabe
        AND $g_inAssignment_table.$g_assignment_table = '$assignment_id'";
    
    $result = $vips_db->executeQuery($sql);
    
    while ($result->next()) {
        $entry = $result->getRow();
        $ID    = $entry['ID'];
        $Name  = addslashes($entry['Name']);  //addslashes!!! important!!!            
        
        #############################################
        # change very old exercises into new format #
        #############################################
        
        $URI = $entry['URI'];
        if ($URI == 'yn_exercise') 
        {
            $Aufgabe = str_replace('<Answer Correct="1"><![CDATA[]]></Answer>',
                '<Answer Correct="1">Ja</Answer><Answer Correct="0">Nein</Answer>',
                $entry['Aufgabe']);
            $Aufgabe = str_replace('<Answer Correct="0"><![CDATA[]]></Answer>',
                '<Answer Correct="0">Ja</Answer><Answer Correct="1">Nein</Answer>',
                $Aufgabe);
            $Aufgabe = addslashes($Aufgabe);
        }
        else 
            $Aufgabe = addslashes($entry['Aufgabe']);  //addslashes!!! important!!!
        ////////////////////////////////////////////////////////////////////////////
        
        $Userid = $g_outlet->userID;
        
        $sql = "INSERT INTO vips_aufgabe
            SET Name    = '$Name',
                Aufgabe = '$Aufgabe',
                URI     = '$URI',
                Userid  = '$Userid'
            ";
        $vips_db->executeUpdate($sql);
        $vips_idgen = $vips_db->getIdGenerator();
        $new_exercise_id = $vips_idgen->getId();
        
        ####################################################
        # copy and paste information in inAssignment_table #
        ####################################################
        
        $sql = "SELECT * FROM $g_inAssignment_table
            WHERE $g_assignment_table = '$assignment_id'
            AND vips_aufgabe = '$ID'";
        
        $result2 = $vips_db->executeQuery($sql);
        
        if ($result2->next()) {
            $punkte   = $result2->getFloat('Punkte');
            $position = $result2->getInt('Position');
        }
        
        
        $sql = "INSERT INTO $g_inAssignment_table
            SET vips_aufgabe        = '$new_exercise_id',
                $g_assignment_table = '$new_assignment_id',
                Punkte              = '$punkte',
                Bewertet            = 't',
                Position            = '$position'";
        $vips_db->executeUpdate($sql);  
    }
    $message['mode']    = 'msg';
    $message['message'] = 'Das Aufgabenblatt wurde kopiert.';
    $_POST['message']   = $message;
    list_assignments($p_action);
}



/**
 * SHEETS/EXAMS
 * 
 * Displays form to create a new assignment
 * Uses the same template as edit_assignment
 *
 */
function new_assignment($p_action) {
    global $vips_db, $g_outlet, $g_title_s, $g_title_p, $vipsTemplateFactory;

    $t_main = $vipsTemplateFactory->open('edit_assignment');

    // default values
    $t_main->set_attributes(array(
        'picturepath'            => $g_outlet->picturepath,
        'g_title_s'              => $g_title_s,
        'g_title_p'              => $g_title_p,
        'action'                 => $p_action,
        'assignment_name'        => $g_title_s,
        'assignment_description' => '',
        'end_year'               => date('Y'),
        'end_month'              => date('m'),
        'end_day'                => date('d'),
        'end_h'                  => date('H'),
        'end_m'                  => '00',
        'start_year'             => date('Y'),
        'start_month'            => date('m'),
        'start_day'              => date('d'),
        'start_h'                => date('H'),
        'start_m'                => '00',
        'selftest'               => 0,
        'printable'              => 0,
        'ip_range'               => '',
        'mc_auswertung'          => 0,
        'exam_length'            => 0,
        'number_of_exercises'    => 0
    ));

    setlocale(LC_NUMERIC, NULL);
    echo $t_main->render();
    setlocale(LC_NUMERIC, 'C');
}

/**
 * SHEETS/EXAMs
 * 
 * Main page of sheets/exams. 
 * Lists all the assignments (sheets or exams) in the course, grouped by "not yet started",
 * "running" and "finished".
 */
function list_assignments($p_action) {
        global $vips_db,
               $g_outlet,
               $g_title_p,
               $g_title_s,
               $g_assignment_table,
               $g_inAssignment_table,
               $vipsTemplateFactory;

        $dialog = getExternalParam('dialog');           //just in case (for header)
        $message = getExternalParam('message');         //just in case (for header)

        $delete_assignment_id   = getExternalParam('delete_assignment_id');
        $delete_assignment_name = getExternalParam('delete_assignment_name');

        $t_list_header  = $vipsTemplateFactory->open('list_assignments_header');

        $t_list_header->set_attributes(array(
                'title' => $g_title_p,
                'picturepath' => $g_outlet->picturepath));
        $t_list_header->set_attribute('dialog', $dialog);
        $t_list_header->set_attribute('message', $message);
        $t_list_header->set_attribute('assignment_id', $delete_assignment_id);
        $t_list_header->set_attribute('assignment_name', $delete_assignment_name);
        $t_list_header->set_attribute('action', $p_action);

        $t_list_footer  = $vipsTemplateFactory->open('list_assignments_footer');
        $t_list_footer->set_attribute('picturepath', $g_outlet->picturepath);
        $t_list_footer->set_attribute('action', $p_action);
        //$t_list_footer->set_attribute('status', $g_outlet->status);


        ##########################
        #       get blocks                       #
        ##########################

        $blocks = array();

        $result = $vips_db->executeQuery("SELECT DISTINCT * FROM vips_block
                                                        WHERE   Kurs = '$g_outlet->courseID'
                                                        ");

        while ($result->next()) {
                $entry = $result->getRow();
                $blocks[] = $entry;
        }

        ######################################
        #   get assignments in this course   #
        ######################################

        $query = "SELECT * FROM $g_assignment_table
                        WHERE
                                Kurs='$g_outlet->courseID'
                        ORDER BY
                                Beginn, Position";

        $result = $vips_db->executeQuery($query);

        $m_not_started_assignments = array();
        $m_running_assignments     = array();
        $m_finished_assignments    = array();
        $m_all_assignments         = array();

        $assignment_ids             = array();

        while ($result->next()) {
                $entry = $result->getRow();
                $assignment_id = $entry['ID'];
                $result2 = $vips_db->executeQuery("SELECT vips_block.* 
                                                    FROM vips_inBlock, 
                                                             vips_block
                                                    WHERE vips_inBlock.assignment_id= '$assignment_id'
                                                    AND   vips_inBlock.block_id     = vips_block.id");
                if ($result2->next()) {
                        $block = $result2->getRow();
                        $entry['block_id'] = $block['id'];
                }
                else $entry['block_id'] = "0";

                $m_start_timestamp = strtotime($entry['Beginn']);
                $m_end_timestamp   = strtotime($entry['Ende']);

                if (time() < $m_start_timestamp)
                    $m_not_started_assignments[] = $entry;
                else if ($m_end_timestamp < time())
                    $m_finished_assignments[]    = $entry;
                else
                    $m_running_assignments[]     = $entry;
                
         }
         
        ##############################################################
        #   get assignments made by lecturer (course-independent)    #
        ##############################################################
        
        $query = "SELECT DISTINCT $g_assignment_table.*,
                         seminare.Name AS course_name,
                         semester_data.name AS sem_name
                  FROM
                         seminare,
                         seminar_user,
                         seminar_user AS lecturer,
                         semester_data,
                         $g_assignment_table
                  WHERE
                         seminare.Seminar_id = seminar_user.Seminar_id
                     AND seminar_user.user_id = lecturer.user_id
                     AND seminar_user.status = 'dozent'
                     AND lecturer.Seminar_id = '$g_outlet->courseID'
                     AND lecturer.status = 'dozent'
                     AND semester_data.beginn = seminare.start_time
                     AND $g_assignment_table.Kurs = seminare.Seminar_id
                  ORDER BY
                         semester_data.beginn DESC,
                         $g_assignment_table.Name";
        $result = $vips_db->executeQuery($query);
         
        while ($result->next()) {
                $m_all_assignments[] = $result->getRow();
        }

        ##################
        #        Templates       #
        ##################
        
        $t_list1                = $vipsTemplateFactory->open('list_assignments_list');
        $t_list2                = $vipsTemplateFactory->open('list_assignments_list');
        $t_list3                = $vipsTemplateFactory->open('list_assignments_list');

        $count_not_started      = count($m_not_started_assignments);
        $count_running          = count($m_running_assignments);
        $count_finished         = count($m_finished_assignments);

        if ($count_not_started) {
                $t_list1->set_attribute('picturepath', $g_outlet->picturepath);
                $t_list1->set_attribute('action',$p_action);
                $t_list1->set_attribute('assignments', $m_not_started_assignments);
                $t_list1->set_attribute('title', "Noch nicht gestartete $g_title_p");
                $t_list1->set_attribute('status_assignment', 'not started');
                $t_list1->set_attribute('blocks', $blocks);
                $t_list1->set_attribute('number_of_assignments', $count_not_started);
        }
        if ($count_running) {
                $t_list2->set_attribute('picturepath', $g_outlet->picturepath);
                $t_list2->set_attribute('action',$p_action);
                $t_list2->set_attribute('assignments', $m_running_assignments);
                $t_list2->set_attribute('title', "Laufende $g_title_p");
                $t_list2->set_attribute('status_assignment', 'running');
                $t_list2->set_attribute('blocks', $blocks);
                $t_list2->set_attribute('number_of_assignments', $count_running);
        }
        if ($count_finished) {
                $t_list3->set_attribute('picturepath', $g_outlet->picturepath);
                $t_list3->set_attribute('action',$p_action);
                $t_list3->set_attribute('assignments', $m_finished_assignments);
                $t_list3->set_attribute('title', "Beendete $g_title_p");
                $t_list3->set_attribute('status_assignment', 'finished');
                $t_list3->set_attribute('blocks', $blocks);
                $t_list3->set_attribute('number_of_assignments', $count_finished);
        }

        $t_list_header->set_attribute('number_of_assignments', $count_not_started + $count_running + $count_finished);
        $t_list_footer->set_attribute('all_assignments', $m_all_assignments);
        $t_list_footer->set_attribute('action', $p_action);
        $t_list_footer->set_attribute('g_title_s', $g_title_s);

        echo $t_list_header->render();

        // list notstarted assignments
        if ($count_not_started && $g_outlet->status != 'student')
                 echo $t_list1->render();
        // list running assignments
        if ($count_running)
                echo $t_list2->render();
        //list finished assignments
        if ($count_finished && $g_outlet->status != 'student')
                 echo $t_list3->render();

        echo $t_list_footer->render();
}


#####################################
#####################################
#                                       #
#               Help Methods            #
#                                       #
#####################################
#####################################


/**
 * EXAM
 * 
 * Calculates the time at which the student has to finish the exam (starting time of the
 * student PLUS length of the exam)
 * If the global endzeitpunkt of the assignment is closer, then this is the Abgabezeitpunkt
 * @return: Abgabezeitpunkt (timestamp)
 */
function getAbgabezeitpunkt($p_assignment_id) {
        global $vips_db,
                   $g_outlet,
                   $g_title_s,
                   $g_assignment_table,
                   $g_inAssignment_table;
        
        $result = $vips_db->executeQuery("SELECT * FROM $g_assignment_table WHERE ID = '$p_assignment_id'");
        $result->next();

        $assignment = $result->getRow();

        $length = $assignment['Dauer'];
        $end    = $assignment['Ende'];
        $start  = $assignment['Beginn'];
        $now    = date("Y-m-d H:i:s");

        $remainingMinutes = 0;
        
        $studentStart = $now;
        
        /*********************************************************************/
        /* Query database table "AufgabenZeiten" for earliest time entry for
        /* user in this exercise_sheet.
        /*
        /* Kurs, vips_klausur and user_id make up the primary key together.
        /*********************************************************************/
        
        $result = $vips_db->executeQuery("SELECT Beginn FROM vips_aufgaben_zeit
                                WHERE   Kurs                    = '$g_outlet->courseID'
                                AND     $g_assignment_table     = '$p_assignment_id'
                                AND     user_id                 = '$g_outlet->userID'
                                ORDER BY Beginn");
        
        if ($result->next())
        {
                $user_time = $result->getRow(); 
                $studentStart = $user_time['Beginn'];
        }
        
        $lengthSec = $length * 60;  //length in seconds
        $abgabeSec = strtotime($studentStart) + $lengthSec;
        
        $abgabe = min(strtotime($end), $abgabeSec);  // timestamp!
        
        return $abgabe;
}

/**
 * EXAM
 * 
 * Calculates the remaining minutes for a student
 * TODO: avoid code duplication with getAbgabezeitpunkt()
 * 
 * @return: remaining minutes of the exam
 * 
 */
function getRemainingMinutes($p_assignment_id) {
        global $vips_db,
                   $g_outlet,
                   $g_title_s,
                   $g_assignment_table,
                   $g_inAssignment_table;

        $result = $vips_db->executeQuery("SELECT * FROM $g_assignment_table WHERE ID = '$p_assignment_id'");
        $result->next();

        $assignment = $result->getRow();

        $m_length       = $assignment['Dauer'];
        $m_end          = $assignment['Ende'];
        $m_start        = $assignment['Beginn'];
        $m_now          = date("Y-m-d H:i:s");

        $m_remainingMinutes = 0;

        /* also show remaining minutes for not yet started exams */
        if($m_now < $m_end /* && $m_now > $m_start */) {

                /*********************************************************************/
                /* Query database table "AufgabenZeiten" for earliest time entry for
                /* user in this exercise_sheet.
                /*
                /* Kurs, vips_klausur and user_id make up the primary key together.
                /*********************************************************************/

                $result = $vips_db->executeQuery("SELECT Beginn FROM vips_aufgaben_zeit
                                        WHERE   Kurs                    = '$g_outlet->courseID'
                                        AND     $g_assignment_table     = '$p_assignment_id'
                                        AND     user_id                 = '$g_outlet->userID'
                                        ORDER BY Beginn");

                if ($result->next())
                        $user_start = $result->get('Beginn');

                $m_remainingMinutes = clip_remaining_time($user_start, $m_length, $m_end);
        }

        return $m_remainingMinutes;
}

/**
 * Clips remaining time to end of exam or test.
 *
 * @param user_start    string  start time of exam (date format)
 * @param exam_length   int     maximum permitted length (minutes)
 * @param exam_end      string  end of exam (date format)
 */
function clip_remaining_time ($user_start, $exam_length, $exam_end)
{
        $time = time();
        $remaining_time = $exam_length * 60;

        if (isset($user_start))
        {
                $elapsed_time = $time - strtotime($user_start);
                $remaining_time -= $elapsed_time;
        }

        // clip remaining time if necessary
        $remaining_time = min($remaining_time, strtotime($exam_end) - $time);

        return ceil($remaining_time / 60);
}

/**
 * EXAM
 * 
 * Checks whether the IP address of the user is listed among the IP addresses given by
 * the lecturer (if applicable)
 * 
 */

function ip_permitted($p_assignment_id) {
        global $vips_db,
                   $g_outlet,
                   $g_title_s,
                   $g_assignment_table,
                   $g_inAssignment_table;


        $result = $vips_db->executeQuery("SELECT IPBereich
                                        FROM  $g_assignment_table
                                        WHERE ID = '$p_assignment_id'");
        $result->next();
        $assignment = $result->getRow();
        $m_exercise_ip_rangeSpaceSeparatedList  = $assignment["IPBereich"];

        /**************************************************************************/
        /* Explode space separated list into an array and check the resulting single
        /* IPs against the current user IP.
        /* No IP given: user has access.
        /* One or more IPs are given and user IP matches at least one: user has
        /* access.
        /**************************************************************************/

        // Explode seems to hand separator characters, too?!
        // Therefore, this is not too good solution:
        $m_hasAccess = TRUE;
        if(strlen($m_exercise_ip_rangeSpaceSeparatedList) > 0) {

                $m_hasAccess = FALSE;
                $m_exercise_ip_rangeStringArray = explode(" ",$m_exercise_ip_rangeSpaceSeparatedList);
        
                foreach($m_exercise_ip_rangeStringArray AS $m_IP) {
                        trim($m_IP);
                        $own_IP = $_SERVER['REMOTE_ADDR'];
                        if($m_IP != "") {
                                if (strpos($m_IP,"-")) {
                                        $startend       = explode("-",$m_IP);
                                        $start          = $startend[0];
                                        $end            = $startend[1];
                                        if ($_SERVER['REMOTE_ADDR'] >= $start && $_SERVER['REMOTE_ADDR'] <= $end)
                                                $m_hasAccess = TRUE;
                                        
                                }
                                else {
                                        $m_tmp = strpos(" ".$own_IP, $m_IP);
                                        if ($m_tmp == 1) $m_hasAccess = TRUE;
                                }
                        }
                }
        }
        return $m_hasAccess;
}

/**
 * EXAM
 * 
 * @return info on Gesperrtsein of exam
 */
function checkInterrupted($p_assignment_id) {
                global $vips_db,
                   $g_outlet,
                   $g_assignment_table,
                   $g_inAssignment_table;

        $result = $vips_db->executeQuery("SELECT istGesperrt
                                                FROM  $g_assignment_table
                                                WHERE ID = '$p_assignment_id'");
        $result->next();
        $assignment = $result->getRow();
        return $assignment["istGesperrt"];
}

?>
